@@ -1,22 +1,21 @@
-/* $Id: agent_shared.c,v 1.71 2006/05/27 19:45:11 jonz Exp $ */
+/* $Id: agent_shared.c,v 1.86 2011/07/11 22:05:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -37,7 +36,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
-#ifdef HAVE_UNISTD_H_
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #include <pwd.h>
 #endif
@@ -94,12 +93,12 @@ uid_t  __pw_uid;
 
 int initialize_atx(AGENT_CTX *ATX) {
   memset(ATX, 0, sizeof(AGENT_CTX));
-  ATX->training_mode   = DST_DEFAULT;
   ATX->training_buffer = 0;
   ATX->train_pristine  = 0;
   ATX->classification  = DSR_NONE;
   ATX->source          = DSS_NONE;
   ATX->operating_mode  = DSM_PROCESS;
+  ATX->fork            = 1;
   ATX->users           = nt_create (NT_CHAR);
 
   if (ATX->users == NULL) {
@@ -228,10 +227,17 @@ int process_arguments(AGENT_CTX *ATX, int argc, char **argv) {
     }
 #endif
  
+    if (!strcmp (argv[i], "--nofork")) {
+      ATX->fork = 0;
+      continue;
+    }
+
     if (!strncmp (argv[i], "--mode=", 7))
     {
       char *mode = strchr(argv[i], '=')+1;
-      process_mode(ATX, mode);
+      if (process_mode(ATX, mode))
+        return EINVAL;
+      ATX->flags |= DAF_FIXED_TR_MODE;
       continue;
     }
 
@@ -254,9 +260,17 @@ int process_arguments(AGENT_CTX *ATX, int argc, char **argv) {
           return EINVAL;
         }
 
-        if (ATX->trusted)
+        if (ATX->trusted) {
+#endif
+          if (_ds_validate_address(user) == 1) {
+            nt_add (ATX->recipients, user);
+          } else {
+            LOG(LOG_ERR, "Invalid email address: %s", user);
+            return EINVAL;
+          }
+#ifdef TRUSTED_USER_SECURITY
+        }
 #endif
-          nt_add (ATX->recipients, user);
       }
       continue;
     }
@@ -267,22 +281,27 @@ int process_arguments(AGENT_CTX *ATX, int argc, char **argv) {
     {
       if (argv[i] != NULL && strlen (argv[i]) < MAX_USERNAME_LENGTH)
       {
-        char user[MAX_USERNAME_LENGTH];
+        if (strstr(argv[i], "../") != NULL || strstr(argv[i], "..\\") != NULL) {
+          LOG(LOG_ERR, "Illegal username ('../' or '..\\' not allowed in username)");
+          return EINVAL;
+        } else {
+          char user[MAX_USERNAME_LENGTH];
 
-        if (_ds_match_attribute(agent_config, "Broken", "case")) 
-          lc(user, argv[i]);
-        else 
-          strcpy(user, argv[i]);
+          if (_ds_match_attribute(agent_config, "Broken", "case"))
+            lc(user, argv[i]);
+          else
+            strcpy(user, argv[i]);
 
 #ifdef TRUSTED_USER_SECURITY
-        if (!ATX->trusted && strcmp(user, __pw_name)) {
-          LOG(LOG_ERR, ERR_TRUSTED_USER, __pw_uid, __pw_name);
-          return EINVAL;
-        }
+          if (!ATX->trusted && strcmp(user, __pw_name)) {
+            LOG(LOG_ERR, ERR_TRUSTED_USER, __pw_uid, __pw_name);
+            return EINVAL;
+          }
 
-        if (ATX->trusted)
+          if (ATX->trusted)
 #endif
-          nt_add (ATX->users, user);
+            nt_add (ATX->users, user);
+        }
       }
       continue;
     }
@@ -427,12 +446,10 @@ int process_arguments(AGENT_CTX *ATX, int argc, char **argv) {
     if (!strcmp (argv[i], "--version"))
     {
       printf ("\nDSPAM Anti-Spam Suite %s (agent/library)\n\n", VERSION);
-      printf ("Copyright (c) 2002-2006 Jonathan A. Zdziarski\n");
-      printf ("http://dspam.nuclearelephant.com\n\n");
-      printf ("DSPAM may be copied only under the terms of the GNU "
-              "General Public License,\n");
-      printf ("a copy of which can be found with the DSPAM distribution "
-              "kit.\n\n");
+      printf ("Copyright (C) 2002-2011 DSPAM Project\n");
+      printf ("http://dspam.sourceforge.net.\n\n");
+      printf ("DSPAM may be copied only under the terms of the GNU Affero General Public\n");
+      printf ("License, a copy of which can be found with the DSPAM distribution kit.\n\n");
 #ifdef TRUSTED_USER_SECURITY
       if (ATX->trusted) {
 #endif
@@ -571,9 +588,12 @@ int apply_defaults(AGENT_CTX *ATX) {
 
   /* Training mode */
 
-  if (ATX->training_mode == DST_DEFAULT) {
+  if (!(ATX->flags & DAF_FIXED_TR_MODE)) {
     char *v = _ds_read_attribute(agent_config, "TrainingMode");
-    process_mode(ATX, v);
+    if (process_mode(ATX, v)) {
+      LOG(LOG_ERR, ERR_AGENT_NO_TR_MODE);
+      return EINVAL;
+    }
   }
 
   /* Default delivery agent */
@@ -590,9 +610,18 @@ int apply_defaults(AGENT_CTX *ATX) {
 #endif
       strcpy(key, "TrustedDeliveryAgent");
 
-    if (_ds_read_attribute(agent_config, key)) {
+    char *value = _ds_read_attribute(agent_config, key);
+
+    if (value) {
+      char *trimmed_value = ALLTRIM(strdup(value));
+      if (trimmed_value && *trimmed_value == '\0') {
+        LOG(LOG_ERR, ERR_AGENT_NO_AGENT, key);
+        free(trimmed_value);
+        return EINVAL;
+      }
+      if (trimmed_value) free(trimmed_value);
       char fmt[sizeof(ATX->mailer_args)];
-      snprintf(fmt, sizeof(fmt), "%s ", _ds_read_attribute(agent_config, key));
+      snprintf(fmt, sizeof(fmt), "%s ", value);
 #ifdef TRUSTED_USER_SECURITY
       if (ATX->trusted)
 #endif
@@ -667,12 +696,6 @@ int check_configuration(AGENT_CTX *ATX) {
     return EINVAL;
   }
 
-  if (ATX->training_mode == DST_DEFAULT)
-  {
-    LOG(LOG_ERR, ERR_AGENT_NO_TR_MODE);
-    return EINVAL;
-  }
-
   if (!_ds_match_attribute(agent_config, "ParseToHeaders", "on")) {
 
     if (ATX->users->items == 0)
@@ -731,13 +754,6 @@ buffer * read_stdin(AGENT_CTX *ATX) {
         }
       }
 
-      /* Quote message termination characters, that could truncate messages */
-      if (buf[0] == '.' && buf[1] < 32) {
-        char x[sizeof(buf)];
-        snprintf(x, sizeof(x), ".%s", buf);
-        strcpy(buf, x);
-      }
-  
       /*
        *  Don't include first line of message if it's a quarantine header added
        *  by dspam at time of quarantine 
@@ -829,71 +845,71 @@ bail:
 int process_parseto(AGENT_CTX *ATX, const char *buf) {
   char *y = NULL;
   char *x;
+  char *h = NULL;
+  char *buffer;
+  char *ptrptr;
 
-  if (!buf) 
+  if (!buf || strncmp(buf+2,":",1) != 0)
     return EINVAL;
 
-  x = strstr(buf, "<spam-");
-  if (!x)
-    x = strstr(buf, " spam-");
-  if (!x)
-    x = strstr(buf, ":spam-");
-  if (!x)
-    x = strstr(buf, "<spam@");
-  if (!x)
-    x = strstr(buf, " spam@");
-  if (!x)
-    x = strstr(buf, ":spam@");
-
-  if (x != NULL) {
-    y = strdup(x+6);
-
-    if (_ds_match_attribute(agent_config, "ChangeModeOnParse", "on"))
-    {
-      ATX->classification = DSR_ISSPAM;
-      ATX->source = DSS_ERROR;
-    }
-  } else {
-
-    x = strstr(buf, "<notspam-");
-    if (!x)
-      x = strstr(buf, " notspam-");
-    if (!x)
-      x = strstr(buf, ":notspam-");
-    if (!x)
-      x = strstr(buf, "<notspam@");
-    if (!x)
-      x = strstr(buf, " notspam@");
-    if (!x)
-      x = strstr(buf, ":notspam@");
-
-    if (x && strlen(x) >= 9) {
-      y = strdup(x+9);
-
-      if (_ds_match_attribute(agent_config, "ChangeModeOnParse", "on"))
-      {
-        ATX->classification = DSR_ISINNOCENT;
+  buffer = strdup (buf+3);
+  h = strtok_r (buffer, "\n", &ptrptr);
+  while (h != NULL) {
+    /* check for spam alias */
+    x = strstr(h, "<spam-");
+    if (!x) x = strstr(h, " spam-");
+    if (!x) x = strstr(h, "\tspam-");
+    if (!x) x = strstr(h, ",spam-");
+    if (!x) x = strstr(h, ":spam-");
+    if (x != NULL) {
+      y = strdup(x+6);
+      if (_ds_match_attribute(agent_config, "ChangeModeOnParse", "on")) {
+        ATX->classification = DSR_ISSPAM;
         ATX->source = DSS_ERROR;
       }
+    } else {
+      /* check for nonspam alias */
+      x = strstr(h, "<notspam-");
+      if (!x) x = strstr(h, " notspam-");
+      if (!x) x = strstr(h, "\tnotspam-");
+      if (!x) x = strstr(h, ",notspam-");
+      if (!x) x = strstr(h, ":notspam-");
+      if (x && strlen(x) >= 9) {
+        y = strdup(x+9);
+        if (_ds_match_attribute(agent_config, "ChangeModeOnParse", "on")) {
+          ATX->classification = DSR_ISINNOCENT;
+          ATX->source = DSS_ERROR;
+        }
+      }
+    }
+    /* do not continue if we found a spam/nonspam alias */
+    if (y) break;
+
+    /* get next line from 'To' header */
+    h = strtok_r (NULL, "\n", &ptrptr);
+    if (h && h[0] != 32 && h[0] != 9) {
+      /* we are not any more in the 'To' header */
+      break;
     }
   }
 
+  free (buffer);
+
   if (y && (_ds_match_attribute(agent_config,
                                 "ChangeUserOnParse", "on") ||
             _ds_match_attribute(agent_config,
                                "ChangeUserOnParse", "full") ||
             _ds_match_attribute(agent_config,
-                                "ChangeUserOnParse", "user"))) 
+                                "ChangeUserOnParse", "user")))
   {
-    char *ptrptr;
     char *z;
 
     if (_ds_match_attribute(agent_config, 
                             "ChangeUserOnParse", "full"))
     {
-      z = strtok_r(y, "> \n", &ptrptr);
+      z = strtok_r(y, ">, \t\r\n", &ptrptr);
     } else {
-      if (!strstr(x, "spam@"))
+      if (strstr(x, "@"))
         z = strtok_r(y, "@", &ptrptr);
       else
         z = NULL;
@@ -908,8 +924,23 @@ int process_parseto(AGENT_CTX *ATX, const char *buf) {
       }
       nt_add (ATX->users, z);
     }
-    free(y);
   }
 
+  if (y) free(y);
   return 0;
 }
+
+int
+init_pwent_cache(void)
+{
+  struct passwd *pwent;
+  pwent = getpwuid(getuid());
+  if (pwent == NULL) {
+    return 0;
+  }
+  else {
+     __pw_name = strdup(pwent->pw_name);
+     __pw_uid  = pwent->pw_uid;
+  }
+  return 1;
+}
