@@ -1,22 +1,21 @@
-/* $Id: bnr.c,v 1.28 2006/05/13 01:12:59 jonz Exp $ */
+/* $Id: bnr.c,v 1.32 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -132,8 +131,8 @@ int bnr_instantiate(BNR_CTX *BTX) {
   node_list = c_bnr_list_first(BTX->stream, &c_list);
   while(node_list != NULL) {
     
-    for(i=0;i<BNR_SIZE-1;i++) {
-      previous_bnr_probs[i] = previous_bnr_probs[i+1];
+    for(i=1;i<BNR_SIZE;i++) {
+      previous_bnr_probs[i-1] = previous_bnr_probs[i];
     }
 
     previous_bnr_probs[BNR_SIZE-1] = _bnr_round(node_list->value);
@@ -272,9 +271,9 @@ int bnr_finalize(BNR_CTX *BTX) {
   while(node_list != NULL) {
     float pattern_value;
 
-    for(i=0;i<BNR_SIZE-1;i++) {
-      previous_bnr_probs[i] = previous_bnr_probs[i+1];
-      previous_bnr_tokens[i] = previous_bnr_tokens[i+1];
+    for(i=1;i<BNR_SIZE;i++) {
+      previous_bnr_probs[i-1] = previous_bnr_probs[i];
+      previous_bnr_tokens[i-1] = previous_bnr_tokens[i];
     }
 
     previous_bnr_probs[BNR_SIZE-1] = _bnr_round(node_list->value);
