@@ -1,22 +1,21 @@
-/* $Id: client.c,v 1.62 2006/05/25 15:36:39 jonz Exp $ */
+/* $Id: client.c,v 1.691 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -56,6 +55,7 @@
 #endif
 #include <stdio.h>
 #include <stdlib.h>
+#include <sysexits.h>
 
 #include "client.h"
 #include "dspam.h"
@@ -119,7 +119,7 @@ int client_process(AGENT_CTX *ATX, buffer *message) {
     }
 
     if (client_getcode(&TTX, err, sizeof(err))!=LMTP_OK) {
-      STATUS(err);
+      STATUS("%s", err);
       goto QUIT;
     }
 
@@ -132,22 +132,63 @@ int client_process(AGENT_CTX *ATX, buffer *message) {
     goto BAIL;
 
   if (client_getcode(&TTX, err, sizeof(err))!=LMTP_DATA) {
-    STATUS(err);
+    STATUS("%s", err);
     goto QUIT;
   }
 
   i = 0;
   msglen = strlen(message->data);
   while(i<msglen) {
-    int r = send(TTX.sockfd, message->data+i, msglen - i, 0);
-    if (r <= 0) {
-      STATUS(ERR_CLIENT_SEND_FAILED);
-      goto BAIL;
+    int r;
+    int t;
+    int buflen;
+
+    /*
+     * fill buf with partial msg, replacing \n with \r\n
+     * and do dot stuffing, if needed.
+     */
+    buflen = 0;
+    while ((size_t)buflen < (sizeof(buf) - 4) && i < msglen) {
+      if (i > 0) {
+        if (message->data[i] == '\n') {
+          /* only replace \n and not \r\n */
+          if (message->data[i - 1] != '\r') {
+            buf[buflen] = '\r';
+            buflen++;
+          }
+          /* take care of dot stuffing \n */
+          if (message->data[i + 1] && message->data[i + 1] == '.') {
+            buf[buflen] = '\n';
+            buflen++;
+            buf[buflen] = '.';
+            buflen++;
+            buf[buflen] = '.';
+            buflen++;
+            i += 2;
+            continue;
+          }
+        }
+      }
+
+      buf[buflen] = message->data[i];
+      buflen++;
+      i++;
+    }
+
+    /* send buf */
+    t = 0;
+    while (t < buflen) {
+      r = send(TTX.sockfd, buf+t, buflen - t, 0);
+      if (r <= 0) {
+        LOG(LOG_ERR, ERR_CLIENT_SEND_FAILED);
+        STATUS(ERR_CLIENT_SEND_FAILED);
+        goto BAIL;
+      }
+      t += r;
     }
-    i += r;
   }
 
-  if (message->data[strlen(message->data)-1]!= '\n') {
+  if (message->data[msglen-1]!= '\n') {
     if (send_socket(&TTX, "")<=0) {
      STATUS(ERR_CLIENT_SEND_FAILED);
      goto BAIL;
@@ -188,16 +229,12 @@ int client_process(AGENT_CTX *ATX, buffer *message) {
         }
       } else {
         printf("%s\n", line);
-        if (ATX->flags & DAF_SUMMARY)
-          break;
       } 
       free(line);
       line = client_getline(&TTX, 300);
       if (line) chomp(line);
     }
     free(line);
-    if (line == NULL)
-      goto BAIL;
   } else {
     for(i=0;i<ATX->users->items;i++) {
       char *input = client_getline(&TTX, 300);
@@ -205,7 +242,6 @@ int client_process(AGENT_CTX *ATX, buffer *message) {
       int code = 500;
 
       if (!input) {
-        exitcode = EFAILURE;
         goto BAIL;
       }
       x = strtok(input, " ");
@@ -276,19 +312,24 @@ int client_connect(AGENT_CTX *ATX, int flags) {
   if (flags & CCF_DELIVERY) {
     host = _ds_read_attribute(agent_config, "DeliveryHost");
 
+    if (_ds_read_attribute(agent_config, "DeliveryPort"))
+      port = atoi(_ds_read_attribute(agent_config, "DeliveryPort"));
+
     if (ATX->recipient && ATX->recipient[0]) {
       char *domain = strchr(ATX->recipient, '@');
       if (domain) {
         char key[128];
-        snprintf(key, sizeof(key), "DeliveryHost.%s", domain+1);
+        char lcdomain[strlen(ATX->recipient)];
+        lc(lcdomain, domain+1);
+        snprintf(key, sizeof(key), "DeliveryHost.%s", lcdomain);
         if (_ds_read_attribute(agent_config, key))
           host = _ds_read_attribute(agent_config, key);
+        snprintf(key, sizeof(key), "DeliveryPort.%s", lcdomain);
+        if (_ds_read_attribute(agent_config, key))
+          port = atoi(_ds_read_attribute(agent_config, key));
       }
     }
 
-    if (_ds_read_attribute(agent_config, "DeliveryPort"))
-      port = atoi(_ds_read_attribute(agent_config, "DeliveryPort"));
-
     if (host && host[0] == '/') 
       domain = 1;
 
@@ -319,7 +360,7 @@ int client_connect(AGENT_CTX *ATX, int flags) {
     LOGDEBUG(INFO_CLIENT_CONNECTING, host, 0);
     if(connect(sockfd, (struct sockaddr *)&saun, addr_len)<0) {
       LOG(LOG_ERR, ERR_CLIENT_CONNECT_SOCKET, host, strerror(errno));
-      STATUS(strerror(errno));
+      STATUS("%s", strerror(errno));
       close(sockfd);
       return EFAILURE;
     }
@@ -336,7 +377,7 @@ int client_connect(AGENT_CTX *ATX, int flags) {
     LOGDEBUG(INFO_CLIENT_CONNECTING, host, port);
     if(connect(sockfd, (struct sockaddr *)&addr, addr_len)<0) {
       LOG(LOG_ERR, ERR_CLIENT_CONNECT_HOST, host, port, strerror(errno));
-      STATUS(strerror(errno));
+      STATUS("%s", strerror(errno));
       close(sockfd);
       return EFAILURE;
     }
@@ -374,7 +415,7 @@ int client_authenticate(THREAD_CTX *TTX, const char *mode) {
   pmode[0] = 0;
   if (mode) {
     int pos = 0, cpos = 0;
-    for(;mode[cpos]&&pos<(sizeof(pmode)-1);cpos++) {
+    for(;mode[cpos]&&(size_t)pos<(sizeof(pmode)-1);cpos++) {
       if (mode[cpos] == '"') {
         pmode[pos] = '\\';
         pos++;
@@ -509,7 +550,7 @@ int client_parsecode(char *error) {
  */
 
 int client_getcode(THREAD_CTX *TTX, char *err, size_t len) {
-  char *inp, *ptr, *ptrptr;
+  char *inp, *ptr, *ptrptr = NULL;
   int i;
 
   inp = client_getline(TTX, 300);
@@ -643,7 +684,10 @@ int send_socket(THREAD_CTX *TTX, const char *text) {
   }
 
   r = send(TTX->sockfd, "\r\n", 2, 0);
-  return i+2;
+  if (r > 0) {
+    i += r;
+  }
+  return i;
 }
 
 /*
@@ -678,8 +722,10 @@ int deliver_socket(AGENT_CTX *ATX, const char *msg, int proto) {
   char *ident = _ds_read_attribute(agent_config, "DeliveryIdent");
   int exitcode = EFAILURE;
   int msglen, code;
+  int buflen;
   char *inp;
   int i;
+  int size_extension = 0;
 
   err[0] = 0;
 
@@ -700,7 +746,7 @@ int deliver_socket(AGENT_CTX *ATX, const char *msg, int proto) {
   inp = client_expect(&TTX, LMTP_GREETING, err, sizeof(err));
   if (inp == NULL) {
     LOG(LOG_ERR, ERR_CLIENT_ON_GREETING, err);
-    STATUS(err);
+    STATUS("%s", err);
     goto BAIL;
   }
   free(inp);
@@ -715,9 +761,50 @@ int deliver_socket(AGENT_CTX *ATX, const char *msg, int proto) {
     goto BAIL;
   }
 
-  /* MAIL FROM */
+  /* Check for SIZE extension */
 
-  inp = client_expect(&TTX, LMTP_OK, err, sizeof(err));
+  if (proto == DDP_LMTP) {
+    char *dup, *ptr, *ptrptr;
+    inp = client_getline(&TTX, 300);
+    while(inp != NULL) {
+      code = 0;
+      dup = strdup(inp);
+      if (!dup) {
+        free(inp);
+        LOG(LOG_CRIT, ERR_MEM_ALLOC);
+        LOG(LOG_ERR, ERR_CLIENT_INVALID_RESPONSE, "LHLO", ERR_MEM_ALLOC);
+        STATUS("LHLO: %s", ERR_MEM_ALLOC);
+        goto QUIT;
+      }
+      if (!strcmp(dup, "250-SIZE") || (!strncmp(dup, "250-SIZE", 8) && strlen(dup)>=8 && isspace(dup[8]))) {
+        free(inp);
+        free(dup);
+        size_extension = 1;
+        inp = client_expect(&TTX, LMTP_OK, err, sizeof(err));
+        break;
+      } else if (strncmp(dup, "250-", 4)) {
+        ptr = strtok_r(dup, " ", &ptrptr);
+        if (ptr)
+          code = atoi(ptr);
+        if (code == LMTP_OK) {
+          ptr = strtok_r(NULL, " ", &ptrptr);
+          if (ptr && !strcmp(ptr, "SIZE"))
+            size_extension = 1;
+        }
+        free(dup);
+        if (code == LMTP_OK) {
+          err[0] = 0;
+          break;
+        }
+        LOG(LOG_WARNING, ERR_CLIENT_RESPONSE_CODE, code, inp);
+      }
+      strlcpy(err, inp, sizeof(err));
+      free(inp);
+      inp = client_getline(&TTX, 300);
+    }
+  } else {
+    inp = client_expect(&TTX, LMTP_OK, err, sizeof(err));
+  }
   if (inp == NULL) {
     LOG(LOG_ERR, ERR_CLIENT_INVALID_RESPONSE,
       (proto == DDP_LMTP) ? "LHLO" : "HELO", err);
@@ -726,8 +813,10 @@ int deliver_socket(AGENT_CTX *ATX, const char *msg, int proto) {
   }
   free(inp);
 
-  if (proto == DDP_LMTP) {
-    snprintf(buf, sizeof(buf), "MAIL FROM:<%s> SIZE=%ld", 
+  /* MAIL FROM */
+
+  if (proto == DDP_LMTP && size_extension == 1) {
+    snprintf(buf, sizeof(buf), "MAIL FROM:<%s> SIZE=%ld",
              ATX->mailfrom, (long) strlen(msg));
   } else {
     snprintf(buf, sizeof(buf), "MAIL FROM:<%s>", ATX->mailfrom);
@@ -791,13 +880,40 @@ int deliver_socket(AGENT_CTX *ATX, const char *msg, int proto) {
   i = 0;
   msglen = strlen(msg);
   while(i<msglen) {
-    int r = send(TTX.sockfd, msg+i, msglen - i, 0);
-    if (r <= 0) {
-      LOG(LOG_ERR, ERR_CLIENT_SEND_FAILED);
-      STATUS(ERR_CLIENT_SEND_FAILED);
-      goto BAIL;
+    int r;
+    int t;
+
+    /* fill buf with partial msg, replacing \n with \r\n */
+    buflen = 0;
+    while ((size_t)buflen < (sizeof(buf) - 2) && i < msglen) {
+      /* only replace \n and not \r\n */
+      if (i > 0 && msg[i] == '\n' && msg[i - 1] != '\r') {
+        buf[buflen] = '\r';
+        buflen++;
+      }
+
+      /* escape dot if first character on line */
+      if (msg[i] == '.' && (i == 0 || msg[i - 1] == '\n')) {
+        buf[buflen] = '.';
+        buflen++;
+      }
+
+      buf[buflen] = msg[i];
+      buflen++;
+      i++;
+    }
+
+    /* send buf */
+    t = 0;
+    while (t < buflen) {
+      r = send(TTX.sockfd, buf+t, buflen - t, 0);
+      if (r <= 0) {
+        LOG(LOG_ERR, ERR_CLIENT_SEND_FAILED);
+        STATUS(ERR_CLIENT_SEND_FAILED);
+        goto BAIL;
+      }
+      t += r;
     }
-    i += r;
   }
 
   if (msg[strlen(msg)-1]!= '\n') {
@@ -819,7 +935,9 @@ int deliver_socket(AGENT_CTX *ATX, const char *msg, int proto) {
   code = client_getcode(&TTX, err, sizeof(err));
   if (code < 200 || code >= 300) {
     LOG(LOG_ERR, ERR_CLIENT_RESPONSE, code, "message data", err);
-    if (code >= 500)
+    if (code >= 400 && code < 500)
+      exitcode = EX_TEMPFAIL;
+    else if (code >= 500)
       exitcode = EINVAL;
     chomp(err);
     STATUS((code >= 500) ? "Fatal: %s" : "Deferred: %s", err);
@@ -835,6 +953,9 @@ int deliver_socket(AGENT_CTX *ATX, const char *msg, int proto) {
 QUIT:
   send_socket(&TTX, "QUIT");
   client_getcode(&TTX, err, sizeof(err));
+  buffer_destroy(TTX.packet_buffer);
+  close(TTX.sockfd);
+  return exitcode;
 
 BAIL:
   LOG(LOG_ERR, ERR_CLIENT_DELIVERY_FAILED);
