@@ -1,22 +1,21 @@
-/* $Id: config_api.c,v 1.17 2006/05/13 01:12:59 jonz Exp $ */
+/* $Id: config_api.c,v 1.21 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -83,10 +82,8 @@ int set_libdspam_attributes(DSPAM_CTX *CTX) {
 
       if (!strncasecmp(t->key, "MySQL", 5)  ||
           !strncasecmp(t->key, "PgSQL", 5)  ||
-          !strncasecmp(t->key, "Ora", 3)    ||
           !strncasecmp(t->key, "SQLite", 6) ||
           !strcasecmp(t->key, "LocalMX")    ||
-          !strncasecmp(t->key, "LDAP", 4)   ||
           !strncasecmp(t->key, "Storage", 7) ||
           !strncasecmp(t->key, "Processor", 9) ||
           !strncasecmp(t->key, "Hash", 4))
