@@ -1,22 +1,21 @@
-/* $Id: daemon.c,v 1.114 2006/06/13 14:43:12 jonz Exp $ */
+/* $Id: daemon.c,v 1.22 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -175,9 +174,14 @@ int daemon_listen(DRIVER_CTX *DTX) {
     memset(&local_addr, 0, sizeof(struct sockaddr_in));
     local_addr.sin_family = AF_INET;
     local_addr.sin_port = htons(port);
-    local_addr.sin_addr.s_addr = INADDR_ANY;
-
-    LOGDEBUG(INFO_DAEMON_BIND, port);
+    if (_ds_read_attribute(agent_config, "ServerHost")) {
+      char *host = _ds_read_attribute(agent_config, "ServerHost");
+      local_addr.sin_addr.s_addr = inet_addr(host);
+      LOGDEBUG(INFO_DAEMON_BIND, host, port);
+    } else {
+      local_addr.sin_addr.s_addr = INADDR_ANY;
+      LOGDEBUG(INFO_DAEMON_BIND, "*", port);
+    }
 
     if (bind(listener, (struct sockaddr *)&local_addr, 
              sizeof(struct sockaddr)) == -1) 
@@ -238,7 +242,12 @@ int daemon_listen(DRIVER_CTX *DTX) {
             } else if (!domain) {
               char buff[32];
               LOGDEBUG("connection id %d from %s.", newfd, 
-                       inet_ntoa_r(remote_addr.sin_addr, buff, sizeof(buff)));
+#ifdef HAVE_INET_NTOA_R_2
+                       inet_ntoa_r(remote_addr.sin_addr, buff)
+#else
+                       inet_ntoa_r(remote_addr.sin_addr, buff, sizeof(buff))
+#endif
+                      );
 #endif
             }
             fcntl(newfd, F_SETFL, O_RDWR);
@@ -395,6 +404,9 @@ void *process_connection(void *ptr) {
     if (daemon_extension(TTX, "DSPAMPROCESSMODE")<=0)
       goto CLOSE;
 
+  if (daemon_extension(TTX, "8BITMIME")<=0)
+    goto CLOSE;
+
   if (daemon_reply(TTX, LMTP_OK, "", "SIZE")<=0)
     goto CLOSE;
 
@@ -556,6 +568,7 @@ void *process_connection(void *ptr) {
              !strncasecmp(cmdline, "RSET", 4)))
       {
         char username[256];
+        char *at = NULL;
 
         if (!strncasecmp(cmdline, "RSET", 4)) {
           snprintf(buf, sizeof(buf), "%d OK", LMTP_OK);
@@ -565,15 +578,24 @@ void *process_connection(void *ptr) {
         }
 
         if (_ds_extract_address(username, cmdline, sizeof(username)) ||
-            username[0] == 0 || username[0] == '-')
+            username[0] == 0 || username[0] == '-' || username[0] == '@')
         {
-          daemon_reply(TTX, LMTP_BAD_CMD, "5.1.2", ERR_LMTP_BAD_RCPT);
-          goto GETCMD;
+          if ((server_mode == SSM_DSPAM) || (server_mode == SSM_STANDARD && _ds_validate_address(username) == 0)) {
+            daemon_reply(TTX, LMTP_BAD_CMD, "5.1.2", ERR_LMTP_BAD_RCPT);
+            goto GETCMD;
+          }
         }
 
         if (_ds_match_attribute(agent_config, "Broken", "case"))
           lc(username, username);
 
+        /* Chop of @.* from the recipient */
+        if (_ds_match_attribute(agent_config, "StripRcptDomain", "on")) {
+          at = strchr(username, '@');
+          if (at != NULL)
+            *at = '\0';
+        }
+
         if (server_mode == SSM_DSPAM) {
           nt_add(ATX->users, username);
         }
@@ -588,6 +610,8 @@ void *process_connection(void *ptr) {
               goto CLOSE;
             }
           }
+          if (at != NULL)
+            *at = '@'; /* always add complete address (user@domain) to recipient list */
           nt_add(ATX->recipients, username);
         }
 
@@ -649,15 +673,17 @@ GETCMD:
     }
 
     /* DATA */
-  
-    if (strncasecmp(cmdline, "DATA", 4)) {
-      if (daemon_reply(TTX, LMTP_BAD_CMD, "5.0.0", "Need DATA Here")<0) 
-        goto CLOSE;
-      input = daemon_expect(TTX, "DATA");
-      if (input == NULL)
-        goto CLOSE;
-      if (RSET(input)) 
-        goto RSET;
+
+    if (cmdline != NULL) {
+      if (strncasecmp(cmdline, "DATA", 4)) {
+        if (daemon_reply(TTX, LMTP_BAD_CMD, "5.0.0", "Need DATA Here")<0)
+          goto CLOSE;
+        input = daemon_expect(TTX, "DATA");
+        if (input == NULL)
+          goto CLOSE;
+        if (RSET(input))
+          goto RSET;
+      }
     }
 
     if (daemon_reply(TTX, LMTP_DATA, "", INFO_LMTP_DATA)<=0)
@@ -730,9 +756,8 @@ GETCMD:
     /* Send a terminating '.' if --stdout in 'dspam' mode */
 
     if (ATX->sockfd_output) {
-      if (!(ATX->flags & DAF_SUMMARY))
-        if (send_socket(TTX, ".")<=0)
-          goto CLOSE;
+      if (send_socket(TTX, ".")<=0)
+        goto CLOSE;
 
     /* Otherwise, produce standard delivery results */
 
@@ -765,12 +790,12 @@ GETCMD:
         } 
         else
         {
-          if (result->exitcode == ERC_PERMANENT_DELIVERY) {
+          if (result != NULL && result->exitcode == ERC_PERMANENT_DELIVERY) {
             snprintf(buf, sizeof(buf), "%d 5.3.0 <%s> %s",
                  LMTP_FAILURE, (char *) node_nt->ptr,
                  (result->text[0]) ? result->text : "Permanent error occured");
           } else {
-            if (result->text[0]) {
+            if (result != NULL && result->text[0]) {
               snprintf(buf, sizeof(buf),
                        "%d 4.3.0 <%s> %s",
                        LMTP_TEMP_FAIL, (char *) node_nt->ptr, result->text);
@@ -778,7 +803,7 @@ GETCMD:
               snprintf(buf, sizeof(buf),
                        "%d 4.3.0 <%s> Error occured during %s", 
                        LMTP_TEMP_FAIL, (char *) node_nt->ptr,
-                (result->exitcode == ERC_DELIVERY) ? "delivery" : "processing");
+                (result != NULL && result->exitcode == ERC_DELIVERY) ? "delivery" : "processing");
             }
           }
         }
@@ -810,7 +835,7 @@ RSET:
       free(cmdline);
       cmdline = NULL;
       TTX->authenticated = 0;
-      argc = 0;
+      /* argc = 0; */
     }
 
     free(p);
@@ -886,7 +911,7 @@ buffer * read_sock(THREAD_CTX *TTX, AGENT_CTX *ATX) {
     if (strip) {
       size_t len = strlen(buf);
   
-      while (len>1 && buf[len-2]==13) {
+      while (len>1 && buf[len-2]=='\n') {
         buf[len-2] = buf[len-1];
         buf[len-1] = 0;
         len--;
@@ -902,6 +927,15 @@ buffer * read_sock(THREAD_CTX *TTX, AGENT_CTX *ATX) {
           process_parseto(ATX, buf);
       }
 
+      /* remove dot stuffing, if needed */
+      if((buf[0] && buf[0]=='.') && (buf[1] && buf[1]=='.')) {
+        size_t i, len = strlen(buf);
+        for(i=0;i<len;i++){
+          buf[i]=buf[i+1];
+        }
+        buf[len-1]=0;
+      }
+
       if (buffer_cat (message, buf) || buffer_cat(message, "\n"))
       {
         LOG (LOG_CRIT, ERR_MEM_ALLOC);
