@@ -1,22 +1,21 @@
-/* $Id: decode.h,v 1.12 2006/05/13 01:12:59 jonz Exp $ */
+/* $Id: decode.h,v 1.21 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -49,10 +48,10 @@ typedef struct _ds_header_field
 
 /*
  * _ds_message_part
- * 
+ *
  * DESCRIPTION
- *   a message block (or part) within a message. in a single-part message, 
- *   there will be only one block (block 0). in a multipart message, each part 
+ *   a message block (or part) within a message. in a single-part message,
+ *   there will be only one block (block 0). in a multipart message, each part
  *   will be separated into a separte block. the message block consists of:
  *    - a dynamic array of headers (nodetree of ds_header_t's) for the block
  *    - body data (NULL if there is no body)
@@ -81,9 +80,9 @@ typedef struct _ds_message_part
  * DESCRIPTION
  *   the actual message structure, comprised of an array of message blocks.
  *   in a non-multipart email, there will only be one message block (block 0).
- *   in multipart emails, however, the first message block will represent the 
- *   header (with a NULL body_data or something like "This is a multi-part 
- *   message"), and each additional block within the email will be given its 
+ *   in multipart emails, however, the first message block will represent the
+ *   header (with a NULL body_data or something like "This is a multi-part
+ *   message"), and each additional block within the email will be given its
  *   own message_part structure with its own headers, boundary, etc.
  *
  *   embedded multipart messages are not realized by the structure, but can
@@ -92,23 +91,21 @@ typedef struct _ds_message_part
 
 typedef struct _ds_message
 {
-  struct nt *	components; 
+  struct nt *	components;
   int protect;
 } *ds_message_t;
 
 /* adapter dependent functions */
 
-#ifndef NCORE
 char *  _ds_decode_base64       (const char *body);
 char *  _ds_decode_quoted       (const char *body);
-#endif
 
 /* Adapter-independent functions */
 
 ds_message_t _ds_actualize_message (const char *message);
 
-char *  _ds_assemble_message (ds_message_t message);
-char *  _ds_find_header (ds_message_t message, const char *heading, int flags);
+char *  _ds_assemble_message (ds_message_t message, const char *newline);
+char *  _ds_find_header (ds_message_t message, const char *heading);
 
 ds_message_part_t _ds_create_message_part (void);
 ds_header_t        _ds_create_header_field  (const char *heading);
@@ -123,6 +120,7 @@ char *	_ds_decode_block	(ds_message_part_t block);
 int	_ds_encode_block	(ds_message_part_t block, int encoding);
 char *	_ds_encode_base64	(const char *body);
 char *	_ds_encode_quoted	(const char *body);
+char *	_ds_decode_hex8bit	(const char *body);
 int     _ds_decode_headers      (ds_message_part_t block);
 
 int	_ds_push_boundary	(struct nt *stack, const char *boundary);
@@ -130,6 +128,9 @@ int	_ds_match_boundary	(struct nt *stack, const char *buff);
 int     _ds_extract_boundary    (char *buf, size_t size, char *data);
 char *	_ds_pop_boundary	(struct nt *stack);
 
+char *	_ds_strip_html		(const char *html);
+int	_ds_hex2dec		(unsigned char hex);
+
 /* Encoding values */
 
 #define EN_7BIT			0x00
@@ -175,9 +176,5 @@ char *	_ds_pop_boundary	(struct nt *stack);
 #define BP_HEADER		0x00
 #define BP_BODY			0x01
 
-/* Decoding function flags */
-
-#define DDF_ICASE		0x01
-
 #endif /* _DECODE_H */
 
