@@ -1,22 +1,21 @@
-/* $Id: dspamc.c,v 1.12 2006/05/13 01:12:59 jonz Exp $ */
+/* $Id: dspamc.c,v 1.19 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -42,7 +41,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
-#ifdef HAVE_UNISTD_H_
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #include <pwd.h>
 #endif
@@ -95,24 +94,34 @@ char debug_text[1024];
 int
 main (int argc, char *argv[])
 {
-  AGENT_CTX ATX;
-  int exitcode = EXIT_SUCCESS;
-  buffer *message = NULL;       /* input Message */
+  AGENT_CTX ATX;		/* agent configuration */
+  buffer *message = NULL;	/* input Message */
   int agent_init = 0;		/* agent is initialized */
+  int pwent_cache_init = 0;	/* cache for username and uid is initialized */
+  int exitcode = EXIT_SUCCESS;
+
+  srand ((long) time(NULL) ^ (long) getpid());
+  umask (006);			/* rw-rw---- */
 
   setbuf (stdout, NULL);	/* unbuffered output */
 #ifdef DEBUG
   DO_DEBUG = 0;
 #endif
 
-  srand ((long) time << (long) getpid());
-  umask (006);
-
 #ifndef DAEMON
   LOG(LOG_ERR, ERR_DAEMON_NO_SUPPORT);
   exit(EXIT_FAILURE);
 #endif
 
+  /* Cache my username and uid for trusted user security */
+
+  if (!init_pwent_cache()) {
+    LOG(LOG_ERR, ERR_AGENT_RUNTIME_USER);
+    exitcode = EXIT_FAILURE;
+    goto BAIL;
+  } else
+    pwent_cache_init = 1;
+
   /* Read dspam.conf into global config structure (ds_config_t) */
 
   agent_config = read_config(NULL);
@@ -128,7 +137,7 @@ main (int argc, char *argv[])
     goto BAIL;
   }
 
-  /* Set up agent context to define behavior of processor */
+  /* Set up an agent context to define the behavior of the processor */
 
   if (initialize_atx(&ATX)) {
     LOG(LOG_ERR, ERR_AGENT_INIT_ATX);
@@ -180,6 +189,9 @@ main (int argc, char *argv[])
        _ds_read_attribute(agent_config, "ServerDomainSocketPath")))
   {
     exitcode = client_process(&ATX, message);
+    if (exitcode<0) {
+      LOG(LOG_ERR, ERR_CLIENT_EXIT, exitcode);
+    }
   } else {
     LOG(LOG_ERR, ERR_CLIENT_INVALID_CONFIG);
     exitcode = EINVAL;
@@ -191,12 +203,16 @@ BAIL:
   if (message)
     buffer_destroy(message);
 
-  if (agent_init)
+  if (agent_init) {
     nt_destroy(ATX.users);
+    nt_destroy(ATX.recipients);
+  }
 
   if (agent_config)
     _ds_destroy_config(agent_config);
 
+  if (pwent_cache_init)
+    free(__pw_name);
+
   exit (exitcode);
 }
-
