@@ -1,22 +1,21 @@
-/* $Id: hash.c,v 1.7 2006/05/13 01:12:59 jonz Exp $ */
+/* $Id: hash.c,v 1.75 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -214,11 +213,9 @@ float
 bnr_hash_value (struct bnr_hash *hash, const char *name)
 {
   unsigned long hash_code;
-  struct bnr_hash_node *parent;
   struct bnr_hash_node *node;
 
   hash_code = bnr_hash_hashcode(hash, name);
-  parent = NULL;
   node = hash->tbl[hash_code];
 
   while (node != NULL)
@@ -226,7 +223,6 @@ bnr_hash_value (struct bnr_hash *hash, const char *name)
     if (!strcmp(node->name, name))
       return node->value;
 
-    parent = node;
     node = node->next;
   }
 
@@ -237,14 +233,12 @@ int
 bnr_hash_set (struct bnr_hash *hash, const char *name, float value)
 {
   unsigned long hash_code;
-  struct bnr_hash_node *parent;
   struct bnr_hash_node *node;
 
   if (!name)
     return 0;
 
   hash_code = bnr_hash_hashcode(hash, name);
-  parent = NULL;
   node = hash->tbl[hash_code];
 
   while (node != NULL)
@@ -254,7 +248,6 @@ bnr_hash_set (struct bnr_hash *hash, const char *name, float value)
       return 0;
     }
 
-    parent = node;
     node = node->next;
   }
 
