@@ -1,22 +1,21 @@
-/* $Id: hash_drv.c,v 1.21 2006/05/27 21:00:36 jonz Exp $ */
+/* $Id: hash_drv.c,v 1.296 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -104,7 +103,8 @@ dspam_init_driver (DRIVER_CTX *DTX)
     unsigned long extent_size  = HASH_EXTENT_MAX;
     int pctincrease = 0;
     int flags = HMAP_AUTOEXTEND;
-    int ret, i;
+    int ret;
+    unsigned long i;
 
     if (READ_ATTRIB("HashConnectionCache") && !HashConcurrentUser)
       connection_cache = strtol(READ_ATTRIB("HashConnectionCache"), NULL, 0);
@@ -132,7 +132,7 @@ dspam_init_driver (DRIVER_CTX *DTX)
     }
 
     if (READ_ATTRIB("HashMaxSeek"))
-       max_seek = strtol(READ_ATTRIB("HashMaxSeek"), NULL, 0);
+      max_seek = strtol(READ_ATTRIB("HashMaxSeek"), NULL, 0);
 
     /* Connection array (just one single connection for hash_drv) */
     DTX->connections = calloc(1, sizeof(struct _ds_drv_connection *) * connection_cache);
@@ -189,9 +189,9 @@ dspam_init_driver (DRIVER_CTX *DTX)
 memerr:
   if (DTX) {
     if (DTX->connections) {
-      int i;
+      unsigned long i;
       for(i=0;i<connection_cache;i++) {
-        if (DTX->connections[i]) 
+        if (DTX->connections[i])
           free(DTX->connections[i]->dbh);
         free(DTX->connections[i]);
       }
@@ -216,14 +216,14 @@ dspam_shutdown_driver (DRIVER_CTX *DTX)
     HashConcurrentUser = READ_ATTRIB("HashConcurrentUser");
 
     if (DTX->flags & DRF_STATEFUL) {
-      hash_drv_map_t map;
-      int connection_cache = 1, i;
+      int connection_cache = 1;
 
-     if (READ_ATTRIB("HashConnectionCache") && !HashConcurrentUser)
+      if (READ_ATTRIB("HashConnectionCache") && !HashConcurrentUser)
         connection_cache = strtol(READ_ATTRIB("HashConnectionCache"), NULL, 0);
 
       LOGDEBUG("unloading hash database from memory");
       if (DTX->connections) {
+        int i;
         for(i=0;i<connection_cache;i++) {
           LOGDEBUG("unloading connection object %d", i);
           if (DTX->connections[i]) {
@@ -232,7 +232,7 @@ dspam_shutdown_driver (DRIVER_CTX *DTX)
             }
             else {
               pthread_rwlock_destroy(&DTX->connections[i]->rwlock);
-              map = (hash_drv_map_t) DTX->connections[i]->dbh;
+              hash_drv_map_t map = (hash_drv_map_t) DTX->connections[i]->dbh;
               if (map)
                 _hash_drv_close(map);
             }
@@ -263,7 +263,7 @@ _hash_drv_lock_get (
 
   s->lock = fopen(filename, "a");
   if (s->lock == NULL) {
-    LOG(LOG_ERR, ERR_IO_FILE_OPEN, filename, strerror(errno));
+    LOG(LOG_ERR, ERR_IO_FILE_WRITE, filename, strerror(errno));
     return EFAILURE;
   }
   r = _ds_get_fcntl_lock(fileno(s->lock));
@@ -294,6 +294,57 @@ _hash_drv_lock_free (
   return r;
 }
 
+FILE*
+_hash_tools_lock_get (const char *cssfilename)
+{
+  char filename[MAX_FILENAME_LENGTH];
+  char *pPeriod;
+  int r;
+  FILE* lockfile = NULL;
+
+  if (cssfilename == NULL)
+    return NULL;
+  pPeriod = strrchr(cssfilename, '.');
+  if (pPeriod == NULL || strcmp(pPeriod + 1, "css") || (size_t)(pPeriod - cssfilename + 5) >= sizeof(filename))
+    return NULL;
+  strncpy(filename, cssfilename, pPeriod - cssfilename + 1);
+  strcpy(filename + (pPeriod - cssfilename + 1), "lock");
+  _ds_prepare_path_for(filename);
+
+  lockfile = fopen(filename, "a");
+  if (lockfile == NULL) {
+    LOG(LOG_ERR, ERR_IO_FILE_OPEN, filename, strerror(errno));
+    return NULL;
+  }
+  r = _ds_get_fcntl_lock(fileno(lockfile));
+  if (r) {
+    fclose(lockfile);
+	lockfile = NULL;
+    LOG(LOG_ERR, ERR_IO_LOCK, filename, r, strerror(errno));
+  }
+  return lockfile;
+}
+
+int
+_hash_tools_lock_free (
+  const char *cssfilename,
+  FILE* lockfile)
+{
+  int r;
+
+  if (cssfilename == NULL || lockfile == NULL)
+    return 0;
+
+  r = _ds_free_fcntl_lock(fileno(lockfile));
+  if (!r) {
+    fclose(lockfile);
+  } else {
+    LOG(LOG_ERR, ERR_IO_LOCK_FREE, cssfilename, r, strerror(errno));
+  }
+
+  return r;
+}
+
 int _hash_drv_open(
   const char *filename, 
   hash_drv_map_t map, 
@@ -307,6 +358,7 @@ int _hash_drv_open(
   struct _hash_drv_header header;
   int open_flags = O_RDWR;
   int mmap_flags = PROT_READ + PROT_WRITE;
+  FILE *f;
 
   map->fd = open(filename, open_flags);
 
@@ -317,9 +369,8 @@ int _hash_drv_open(
    */
 
   if (map->fd < 0 && recmaxifnew) {
-    FILE *f;
     struct _hash_drv_spam_record rec;
-    int i;
+    unsigned long i;
 
     memset(&header, 0, sizeof(struct _hash_drv_header));
     memset(&rec, 0, sizeof(struct _hash_drv_spam_record));
@@ -328,19 +379,21 @@ int _hash_drv_open(
 
     f = fopen(filename, "w");
     if (!f) {
-      LOG(LOG_ERR, ERR_IO_FILE_OPEN, filename, strerror(errno));
+      LOG(LOG_ERR, ERR_IO_FILE_WRITE, filename, strerror(errno));
       return EFILE;
     }
 
-    fwrite(&header, sizeof(struct _hash_drv_header), 1, f);
+    if(fwrite(&header, sizeof(struct _hash_drv_header), 1, f)!=1)
+      goto WRITE_ERROR;
     for(i=0;i<header.hash_rec_max;i++)
-      fwrite(&rec, sizeof(struct _hash_drv_spam_record), 1, f);
+      if(fwrite(&rec, sizeof(struct _hash_drv_spam_record), 1, f)!=1)
+        goto WRITE_ERROR;
     fclose(f);
     map->fd = open(filename, open_flags);
   }
 
   if (map->fd < 0) {
-    LOG(LOG_ERR, ERR_IO_FILE_OPEN, filename, strerror(errno));
+    LOG(LOG_ERR, ERR_IO_FILE_WRITE, filename, strerror(errno));
     return EFILE;
   }
 
@@ -352,8 +405,13 @@ int _hash_drv_open(
     return EFAILURE;
   }
 
-  read(map->fd, map->header, sizeof(struct _hash_drv_header));
-  map->file_len = lseek(map->fd, 0, SEEK_END); 
+  if (read(map->fd, map->header, sizeof(struct _hash_drv_header))
+        != sizeof(struct _hash_drv_header)) {
+    free(map->header);
+    close(map->fd);
+    return EFAILURE;
+  }
+  map->file_len = lseek(map->fd, 0, SEEK_END);
 
   map->addr = mmap(NULL, map->file_len, mmap_flags, MAP_SHARED, map->fd, 0);
   if (map->addr == MAP_FAILED) {
@@ -371,6 +429,12 @@ int _hash_drv_open(
   map->flags       = flags;
 
   return 0;
+
+WRITE_ERROR:
+  fclose(f);
+  unlink(filename);
+  LOG(LOG_ERR, ERR_IO_FILE_WRITING, filename, strerror(errno));
+  return EFILE;
 }
 
 int
@@ -387,9 +451,12 @@ _hash_drv_close(hash_drv_map_t map) {
   if (r) {
     LOG(LOG_WARNING, "munmap failed on error %d: %s", r, strerror(errno));
   }
- 
+
   lseek (map->fd, 0, SEEK_SET);
-  write (map->fd, &header, sizeof(struct _hash_drv_header));
+  r = write (map->fd, &header, sizeof(struct _hash_drv_header));
+  if (r < 0) {
+    LOG(LOG_WARNING, "write failed on error %d: %s", r, strerror(errno));
+  }
   close(map->fd);
 
   map->addr = 0;
@@ -404,7 +471,6 @@ _ds_init_storage (DSPAM_CTX * CTX, void *dbh)
 {
   struct _hash_drv_storage *s = NULL;
   hash_drv_map_t map = NULL;
-  int ret;
 
   if (CTX == NULL)
     return EINVAL;
@@ -484,6 +550,7 @@ _ds_init_storage (DSPAM_CTX * CTX, void *dbh)
   {
     char db[MAX_FILENAME_LENGTH];
     int lock_result;
+    int ret;
 
     if (CTX->group == NULL)
       _ds_userdir_path(db, CTX->home, CTX->username, "css");
@@ -527,7 +594,6 @@ _ds_shutdown_storage (DSPAM_CTX * CTX)
   struct _hash_drv_storage *s;
   struct nt_node *node_nt;
   struct nt_c c_nt;
-  int lock_result;
 
   if (!CTX || !CTX->storage)
     return EINVAL;
@@ -554,7 +620,7 @@ _ds_shutdown_storage (DSPAM_CTX * CTX)
   if (!s->dbh_attached) {
     _hash_drv_close(s->map);
     free(s->map);
-    lock_result =
+    int lock_result =
       _hash_drv_lock_free (s, (CTX->group) ? CTX->group : CTX->username);
     if (lock_result < 0)
       return EUNKNOWN;
@@ -694,8 +760,8 @@ _ds_get_spamrecord (
 
   stat->probability   = 0.00000;
   stat->status        = 0;
-  stat->innocent_hits = rec.nonspam;
-  stat->spam_hits     = rec.spam;
+  stat->innocent_hits = rec.nonspam & 0x0fffffff;
+  stat->spam_hits     = rec.spam & 0x0fffffff;
 
   return 0;
 }
@@ -713,6 +779,9 @@ _ds_set_spamrecord (
   rec.nonspam = (stat->innocent_hits > 0) ? stat->innocent_hits : 0;
   rec.spam = (stat->spam_hits > 0) ? stat->spam_hits : 0;
 
+  if(rec.nonspam>0x0fffffff)rec.nonspam=0x0fffffff;
+  if(rec.spam>0x0fffffff)rec.spam=0x0fffffff;
+
   return _hash_drv_set_spamrecord(s->map, &rec, stat->offset);
 }
 
@@ -738,7 +807,12 @@ _ds_set_signature (DSPAM_CTX * CTX, struct _ds_spam_signature *SIG,
     LOG(LOG_ERR, ERR_IO_FILE_WRITE, filename, strerror(errno));
     return EFAILURE;
   }
-  fwrite(SIG->data, SIG->length, 1, file);
+  if(fwrite(SIG->data, SIG->length, 1, file)!=1) {
+    fclose(file);
+    unlink(filename);
+    LOG(LOG_ERR, ERR_IO_FILE_WRITING, filename, strerror(errno));
+    return(EFAILURE);
+  }
   fclose(file);
 
   return 0;
@@ -777,7 +851,13 @@ _ds_get_signature (DSPAM_CTX * CTX, struct _ds_spam_signature *SIG,
     LOG(LOG_ERR, ERR_IO_FILE_OPEN, filename, strerror(errno));
     return EFAILURE;
   }
-  fread(SIG->data, statbuf.st_size, 1, file);
+
+  if (fread(SIG->data, statbuf.st_size, 1, file) != 1) {
+    LOG(LOG_ERR, ERR_IO_FILE_READ, filename, strerror(errno));
+    fclose(file);
+    return EFAILURE;
+  }
+
   SIG->length = statbuf.st_size;
   fclose(file);
   return 0;
@@ -785,6 +865,7 @@ _ds_get_signature (DSPAM_CTX * CTX, struct _ds_spam_signature *SIG,
 
 void *_ds_connect (DSPAM_CTX *CTX)
 {
+  CTX = CTX; /* Keep compiler happy */
   return NULL;
 }
 
@@ -795,6 +876,7 @@ _ds_create_signature_id (DSPAM_CTX * CTX, char *buf, size_t len)
   char digit[6];
   int pid, j;
  
+  CTX = CTX; /* Keep compiler happy */
   pid = getpid ();
   snprintf (session, sizeof (session), "%8lx%d", (long) time (NULL), pid);
  
@@ -849,7 +931,7 @@ _ds_get_nexttoken (DSPAM_CTX * CTX)
     s->offset_header = s->map->addr;
     s->offset_nexttoken = sizeof(struct _hash_drv_header);
     memcpy(&rec,
-           s->map->addr+s->offset_nexttoken, 
+           (void *)((unsigned long) s->map->addr + s->offset_nexttoken),
            sizeof(struct _hash_drv_spam_record));
     if (rec.hashcode)
       _ds_get_spamrecord (CTX, rec.hashcode, &stat);
@@ -867,8 +949,8 @@ _ds_get_nexttoken (DSPAM_CTX * CTX)
       (s->offset_header->hash_rec_max * sizeof(struct _hash_drv_spam_record)))
     { 
       if (s->offset_nexttoken < s->map->file_len) {
-        s->offset_header = s->map->addr + 
-          (s->offset_nexttoken - sizeof(struct _hash_drv_spam_record));
+        s->offset_header = (void *)((unsigned long) s->map->addr + 
+          (s->offset_nexttoken - sizeof(struct _hash_drv_spam_record)));
 
         s->offset_nexttoken += sizeof(struct _hash_drv_header);
         s->offset_nexttoken -= sizeof(struct _hash_drv_spam_record);
@@ -879,7 +961,7 @@ _ds_get_nexttoken (DSPAM_CTX * CTX)
     }
 
     memcpy(&rec,
-           s->map->addr+s->offset_nexttoken, 
+           (void *)((unsigned long) s->map->addr + s->offset_nexttoken), 
            sizeof(struct _hash_drv_spam_record));
     _ds_get_spamrecord (CTX, rec.hashcode, &stat);
   }
@@ -947,38 +1029,40 @@ _ds_get_nextuser (DSPAM_CTX * CTX)
     }
   }
 
-  while ((entry = readdir (dir)) != NULL)
-  {
-    struct stat st;
-    char filename[MAX_FILENAME_LENGTH];
-    snprintf (filename, sizeof (filename), "%s/%s", path, entry->d_name);
-
-    if (!strcmp (entry->d_name, ".") || !strcmp (entry->d_name, ".."))
-      continue;
-
-    if (stat (filename, &st)) {
-      continue;
-    }
-
-    /* push a new directory */
-    if (st.st_mode & S_IFDIR)
+  if (dir != NULL) {
+    while ((entry = readdir (dir)) != NULL)
     {
-      DIR *ndir;
+      struct stat st;
+      char filename[MAX_FILENAME_LENGTH];
+      snprintf (filename, sizeof (filename), "%s/%s", path, entry->d_name);
 
-      ndir = opendir (filename);
-      if (ndir == NULL)
+      if (!strcmp (entry->d_name, ".") || !strcmp (entry->d_name, ".."))
         continue;
-      strlcat (path, "/", sizeof (path));
-      strlcat (path, entry->d_name, sizeof (path));
-      nt_add (s->dir_handles, (void *) ndir);
-      return _ds_get_nextuser (CTX);
-    }
-    else if (strlen(entry->d_name)>4 &&
-      !strncmp ((entry->d_name + strlen (entry->d_name)) - 4, ".css", 4))
-    {
-      strlcpy (user, entry->d_name, sizeof (user));
-      user[strlen (user) - 4] = 0;
-      return user;
+
+      if (stat (filename, &st)) {
+        continue;
+      }
+
+      /* push a new directory */
+      if (st.st_mode & S_IFDIR)
+      {
+        DIR *ndir;
+
+        ndir = opendir (filename);
+        if (ndir == NULL)
+          continue;
+        strlcat (path, "/", sizeof (path));
+        strlcat (path, entry->d_name, sizeof (path));
+        nt_add (s->dir_handles, (void *) ndir);
+        return _ds_get_nextuser (CTX);
+      }
+      else if (strlen(entry->d_name)>4 &&
+        !strncmp ((entry->d_name + strlen (entry->d_name)) - 4, ".css", 4))
+      {
+        strlcpy (user, entry->d_name, sizeof (user));
+        user[strlen (user) - 4] = 0;
+        return user;
+      }
     }
   }
 
@@ -1009,7 +1093,6 @@ _ds_get_nextuser (DSPAM_CTX * CTX)
         s->dir_handles->first = NULL;
       free (node_nt);
       s->dir_handles->items--;
-      prev = node_nt;
       break;
     }
     prev = node_nt;
@@ -1025,12 +1108,15 @@ _ds_get_nextuser (DSPAM_CTX * CTX)
 struct _ds_storage_signature *
 _ds_get_nextsignature (DSPAM_CTX * CTX)
 {
+  CTX = CTX; /* Keep compiler happy */
   return NULL;
 }
 
 int
 _ds_delall_spamrecords (DSPAM_CTX * CTX, ds_diction_t diction)
 {
+  CTX = CTX; /* Keep compiler happy */
+  diction = diction; /* Keep compiler happy */
   return 0;
 }
 
@@ -1041,7 +1127,8 @@ int _hash_drv_autoextend(
 {
   struct _hash_drv_header header;
   struct _hash_drv_spam_record rec;
-  int i;
+  int lastsize;
+  unsigned long i;
 
   _hash_drv_close(map);
 
@@ -1062,10 +1149,26 @@ int _hash_drv_autoextend(
 
   LOGDEBUG("adding extent last: %d(%ld) new: %d(%ld) pctincrease: %1.2f", extents, last_extent_size, extents+1, header.hash_rec_max, (map->pctincrease/100.0));
 
-  lseek (map->fd, 0, SEEK_END);
-  write (map->fd, &header, sizeof(struct _hash_drv_header));
+  lastsize=lseek (map->fd, 0, SEEK_END);
+  if(write (map->fd, &header, sizeof(struct _hash_drv_header))!=sizeof(struct _hash_drv_header)) {
+    if (ftruncate(map->fd, lastsize) < 0) {
+      LOG(LOG_WARNING, "unable to truncate hash file %s: %s",
+          map->filename, strerror(errno));
+    }
+    close(map->fd);
+    LOG(LOG_WARNING, "unable to resize hash. open failed: %s", strerror(errno));
+    return EFAILURE;
+  }
   for(i=0;i<header.hash_rec_max;i++) 
-    write (map->fd, &rec, sizeof(struct _hash_drv_spam_record));
+    if(write (map->fd, &rec, sizeof(struct _hash_drv_spam_record))!=sizeof(struct _hash_drv_spam_record)) {
+      if (ftruncate(map->fd, lastsize) < 0) {
+        LOG(LOG_WARNING, "unable to truncate hash file %s: %s",
+            map->filename, strerror(errno));
+      }
+      close(map->fd);
+      LOG(LOG_WARNING, "unable to resize hash. open failed: %s", strerror(errno));
+      return EFAILURE;
+    }
   close(map->fd);
 
   _hash_drv_open(map->filename, map, 0, map->max_seek, 
@@ -1079,9 +1182,9 @@ unsigned long _hash_drv_seek(
   unsigned long long hashcode,
   int flags)
 {
-  hash_drv_header_t header = map->addr + offset;
+  hash_drv_header_t header = (void *)((unsigned long) map->addr + offset);
   hash_drv_spam_record_t rec;
-  unsigned long long fpos;
+  unsigned long fpos;
   unsigned long iterations = 0;
 
   if (offset >= map->file_len)
@@ -1090,7 +1193,7 @@ unsigned long _hash_drv_seek(
   fpos = sizeof(struct _hash_drv_header) + 
     ((hashcode % header->hash_rec_max) * sizeof(struct _hash_drv_spam_record));
 
-  rec = map->addr + offset + fpos;
+  rec = (void *)((unsigned long) map->addr + offset + fpos);
   while(rec->hashcode != hashcode  &&   /* Match token     */ 
         rec->hashcode != 0         &&   /* Insert on empty */
         iterations < map->max_seek)     /* Max Iterations  */
@@ -1100,7 +1203,7 @@ unsigned long _hash_drv_seek(
 
     if (fpos >= (header->hash_rec_max * sizeof(struct _hash_drv_spam_record)))
       fpos = sizeof(struct _hash_drv_header);
-    rec = map->addr + offset + fpos;
+    rec = (void *)((unsigned long) map->addr + offset + fpos);
   }     
 
   if (rec->hashcode == hashcode) 
@@ -1125,13 +1228,13 @@ _hash_drv_set_spamrecord (
     return EINVAL;
 
   if (map_offset) {
-    rec = map->addr + map_offset;
+    rec = (void *)((unsigned long) map->addr + map_offset);
   } else {
     while(rec_offset <= 0 && offset < map->file_len)
     {
       rec_offset = _hash_drv_seek(map, offset, wrec->hashcode, HSEEK_INSERT);
       if (rec_offset <= 0) {
-        hash_drv_header_t header = map->addr + offset;
+        hash_drv_header_t header = (void *)((unsigned long) map->addr + offset);
         offset += sizeof(struct _hash_drv_header) +
           (sizeof(struct _hash_drv_spam_record) * header->hash_rec_max);
         last_extent_size = header->hash_rec_max;
@@ -1153,7 +1256,7 @@ _hash_drv_set_spamrecord (
       }
     }
   
-    rec = map->addr + offset + rec_offset;
+    rec = (void *)((unsigned long) map->addr + offset + rec_offset);
   }
   rec->hashcode = wrec->hashcode;
   rec->nonspam  = wrec->nonspam;
@@ -1181,7 +1284,7 @@ _hash_drv_get_spamrecord (
   {
     rec_offset = _hash_drv_seek(map, offset, wrec->hashcode, 0);
     if (rec_offset <= 0) {
-      hash_drv_header_t header = map->addr + offset;
+      hash_drv_header_t header = (void *)((unsigned long) map->addr + offset);
       offset += sizeof(struct _hash_drv_header) +
         (sizeof(struct _hash_drv_spam_record) * header->hash_rec_max);
       extents++;
@@ -1192,7 +1295,7 @@ _hash_drv_get_spamrecord (
     return 0;
 
   offset += rec_offset;
-  rec = map->addr + offset;
+  rec = (void *)((unsigned long) map->addr + offset);
   
   wrec->nonspam  = rec->nonspam;
   wrec->spam     = rec->spam;
