@@ -1,22 +1,21 @@
-/* $Id: hash_drv.h,v 1.14 2006/05/27 21:00:36 jonz Exp $ */
+/* $Id: hash_drv.h,v 1.19 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -77,8 +76,8 @@ struct _hash_drv_storage
 typedef struct _hash_drv_spam_record
 {
   unsigned long long hashcode;
-  long nonspam;
-  long spam;
+  unsigned long nonspam;
+  unsigned long spam;
 } *hash_drv_spam_record_t;
 
 int _hash_drv_get_spamtotals 
@@ -88,7 +87,7 @@ int _hash_drv_set_spamtotals
   (DSPAM_CTX * CTX);
 
 int _hash_drv_lock_get (
-  DSPAM_CTX * CTX, 
+  DSPAM_CTX *CTX,
   struct _hash_drv_storage *s, 
   const char *username);
 
@@ -96,6 +95,13 @@ int _hash_drv_lock_free (
   struct _hash_drv_storage *s,
   const char *username);
 
+/* lock variant used by css tools */
+FILE* _hash_tools_lock_get (const char *cssfilename);
+
+int _hash_tools_lock_free (
+  const char *cssfilename,
+  FILE* lockfile);
+
 int _hash_drv_open(
   const char *filename,
   hash_drv_map_t map,
