@@ -1,22 +1,21 @@
-/* $Id: language.h,v 1.25 2006/05/13 01:12:59 jonz Exp $ */
+/* $Id: language.h,v 1.31 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -29,19 +28,21 @@
 
 #define ERR_MEM_ALLOC		"Memory allocation failed"
 #define ERR_IO_FILE_OPEN	"Unable to open file for reading: %s: %s"
+#define ERR_IO_FILE_READ	"Unable to read from file: %s: %s"
 #define ERR_IO_FILE_WRITE	"Unable to open file for writing: %s: %s"
+#define ERR_IO_FILE_WRITING	"Unable to write file: %s: %s"
 #define ERR_IO_FILE_CLOSE	"Unable to close file: %s: %s"
 #define ERR_IO_FILE_RENAME	"Unable to rename file: %s: %s"
 #define ERR_IO_DIR_CREATE	"Unable to create directory: %s: %s"
 #define ERR_IO_LOCK             "Failed to lock file %s: %d: %s"
 #define ERR_IO_LOCK_FREE	"Failed to free lock file %s: %d: %s"
 
-/* LDAP related error codes */
+/* External lookup related error codes */
 
-#define ERR_LDAP_INIT_FAIL	"LDAP initialization failed"
-#define ERR_LDAP_PROTO_VER_FAIL	"LDAP failure: unable to set protocol version"
-#define ERR_LDAP_SEARCH_FAIL	"LDAP search failure"
-#define ERR_LDAP_MISCONFIGURED	"LDAP misconfigured"
+#define ERR_EXT_LOOKUP_INIT_FAIL	"Backend initialization failure"
+#define ERR_EXT_LOOKUP_LDAP_PROTO_VER_FAIL	"Unable to set LDAP protocol version"
+#define ERR_EXT_LOOKUP_SEARCH_FAIL	"Backend search failure"
+#define ERR_EXT_LOOKUP_MISCONFIGURED	"External Lookup misconfigured"
 
 /* Agent error codes */
 
@@ -113,7 +114,7 @@ user, e.g.. root or Trusted User [uid=%d(%s)]"
 
 #define INFO_DAEMON_START	"Daemon process starting"
 #define INFO_DAEMON_EXIT	"Daemon process exiting"
-#define INFO_DAEMON_BIND        "Binding to :%d"
+#define INFO_DAEMON_BIND        "Binding to %s:%d"
 #define INFO_DAEMON_DOMAINSOCK  "Creating local domain socket %s"
 #define INFO_DAEMON_RELOAD      "Reloading configuration"
 
