@@ -1,22 +1,21 @@
-/* $Id: libdspam.c,v 1.162 2006/07/17 00:14:02 jonz Exp $ */
+/* $Id: libdspam.c,v 1.205 2011/07/13 00:51:46 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -76,11 +75,6 @@ void *_drv_handle;
 #include "error.h"
 #include "decode.h"
 #include "language.h"
-#ifdef NCORE
-#include <ncore/ncore.h>
-#include <ncore/types.h>
-#include "ncore_adp.h"
-#endif
 
 #define CHI_S   0.1     /* Chi-Sq Strength */
 #define CHI_X   0.5000  /* Chi-Sq Assumed Probability */
@@ -92,12 +86,6 @@ void *_drv_handle;
 int DO_DEBUG = 0;
 #endif
 
-#ifdef NCORE
-nc_dev_t g_ncDevice;
-
-NC_STREAM_CTX	g_ncDelimiters;
-#endif
-
 /*
  * dspam_init()
  *
@@ -176,18 +164,22 @@ DSPAM_CTX * dspam_create (
   DSPAM_CTX *CTX;
 
   CTX = calloc (1, sizeof (DSPAM_CTX));
-  if (CTX == NULL)
+  if (CTX == NULL) {
+    LOG(LOG_WARNING, "dspam_create: unable to allocate space for classification context");
     return NULL;
+  }
 
   CTX->config = calloc(1, sizeof(struct _ds_config));
   if (CTX->config == NULL) {
+    LOG(LOG_WARNING, "dspam_create: unable to allocate space for classification context configuration");
     LOG(LOG_CRIT, ERR_MEM_ALLOC);
     goto bail;
   }
 
   CTX->config->size = 128;
-  CTX->config->attributes = calloc(1, sizeof(attribute_t)*128);
+  CTX->config->attributes = calloc(1, sizeof(attribute_t)*(CTX->config->size));
   if (CTX->config->attributes == NULL) {
+    LOG(LOG_WARNING, "dspam_create: unable to allocate space for classification context attributes");
     LOG(LOG_CRIT, ERR_MEM_ALLOC);
     goto bail;
   }
@@ -230,12 +222,20 @@ DSPAM_CTX * dspam_create (
   return CTX;
 
 bail:
-  if (CTX->config)
-    _ds_destroy_config(CTX->config->attributes);
-  free(CTX->config);
-  free(CTX->username);
-  free(CTX->group);
-  free(CTX);
+  if (CTX != NULL) {
+    if (CTX->config != NULL) {
+      if (CTX->config->attributes != NULL)
+        _ds_destroy_config(CTX->config->attributes);
+      free(CTX->config);
+    }
+    if (CTX->username != NULL)
+      free(CTX->username);
+    if (CTX->group != NULL)
+      free(CTX->group);
+    if (CTX->home != NULL)
+      free(CTX->home);
+    free(CTX);
+  }
   return NULL;
 }
 
@@ -266,15 +266,17 @@ int dspam_clearattributes (DSPAM_CTX * CTX) {
   if (CTX->config == NULL)
     goto bail;
   CTX->config->size = 128;
-  CTX->config->attributes = calloc(1, sizeof(attribute_t)*128);
+  CTX->config->attributes = calloc(1, sizeof(attribute_t)*(CTX->config->size));
   if (CTX->config->attributes == NULL)
     goto bail;
 
   return 0;
 
 bail:
-  free(CTX->config);
-  CTX->config = NULL;
+  if (CTX->config != NULL) {
+    free(CTX->config);
+    CTX->config = NULL;
+  }
   LOG(LOG_CRIT, ERR_MEM_ALLOC);
   return EUNKNOWN;
 }
@@ -416,9 +418,9 @@ dspam_destroy (DSPAM_CTX * CTX)
   free (CTX->group);
   free (CTX->home);
 
-  if (! CTX->_sig_provided && CTX->signature != NULL)
-  {
-    free (CTX->signature->data);
+  if (! CTX->_sig_provided && CTX->signature != NULL) {
+    if (CTX->signature->data != NULL)
+      free (CTX->signature->data);
     free (CTX->signature);
   }
 
@@ -457,11 +459,10 @@ dspam_process (DSPAM_CTX * CTX, const char *message)
   struct timezone tzp;
 #endif
   buffer *header, *body;
-  int spam_result = 0, is_toe = 0, is_undertrain = 0;
+  int spam_result = 0, is_toe = 0, is_undertrain = 0, retcode = 0;
 
 #ifdef DEBUG
-  if (DO_DEBUG)
-    gettimeofday(&tp1, &tzp);
+  gettimeofday(&tp1, &tzp);
 #endif
 
   if (CTX->signature != NULL)
@@ -475,24 +476,24 @@ dspam_process (DSPAM_CTX * CTX, const char *message)
     return EINVAL;
   }
 
-  if (CTX->algorithms == 0) 
+  if (CTX->algorithms == 0)
   {
     LOG(LOG_WARNING, "No algorithms configured. Use CTX->algorithms and DSA_");
     return EINVAL;
   }
 
-  if (CTX->classification != DSR_NONE && CTX->source == DSR_NONE) 
+  if (CTX->classification != DSR_NONE && CTX->source == DSS_NONE)
   {
     LOG(LOG_WARNING, "A classification requires a source be specified");
     return EINVAL;
   }
 
-  if (CTX->classification == DSR_NONE && CTX->source != DSR_NONE)
+  if (CTX->classification == DSR_NONE && CTX->source != DSS_NONE)
   {
     LOG(LOG_WARNING, "A source requires a classification be specified");
     return EINVAL;
   }
- 
+
   /* Set TOE mode pretrain option if we haven't seen many messages yet */
   if (CTX->training_mode == DST_TOE
   && (CTX->totals.innocent_learned <= 100 || CTX->totals.spam_learned <= 100)
@@ -503,8 +504,8 @@ dspam_process (DSPAM_CTX * CTX, const char *message)
   }
 
   /* Classify only for TOE / NOTRAIN mode setting if data is mature enough */
-  if ( CTX->operating_mode == DSM_PROCESS 
-    && CTX->classification == DSR_NONE 
+  if ( CTX->operating_mode == DSM_PROCESS
+    && CTX->classification == DSR_NONE
     && (CTX->training_mode == DST_TOE || CTX->training_mode == DST_NOTRAIN))
   {
     CTX->operating_mode = DSM_CLASSIFY;
@@ -513,17 +514,13 @@ dspam_process (DSPAM_CTX * CTX, const char *message)
 
   /* A signature has been presented for training; process it */
   /* Non-SPBH Signature */
-  if (CTX->operating_mode == DSM_PROCESS 
-   && CTX->classification != DSR_NONE 
-   && CTX->flags & DSF_SIGNATURE 
+  if (CTX->operating_mode == DSM_PROCESS
+   && CTX->classification != DSR_NONE
+   && CTX->flags & DSF_SIGNATURE
    && (CTX->tokenizer != DSZ_SBPH))
   {
-    int i = _ds_process_signature (CTX);
-    if (is_toe)
-      CTX->operating_mode = DSM_PROCESS;
-    if (is_undertrain)
-      CTX->training_mode = DST_TOE;
-    return i;
+    retcode = _ds_process_signature (CTX);
+    goto restore_mode;
   }
 
   header = buffer_create (NULL);
@@ -533,11 +530,8 @@ dspam_process (DSPAM_CTX * CTX, const char *message)
     LOG (LOG_CRIT, ERR_MEM_ALLOC);
     buffer_destroy (header);
     buffer_destroy (body);
-    if (is_toe)
-      CTX->operating_mode = DSM_PROCESS;
-    if (is_undertrain)
-      CTX->training_mode = DST_TOE;
-    return EUNKNOWN;
+    retcode = EUNKNOWN;
+    goto restore_mode;
   }
 
   /* Parse the message if it hasn't already been by the client app */
@@ -560,44 +554,56 @@ dspam_process (DSPAM_CTX * CTX, const char *message)
   /* If SBPH reclassification, recall and operate on saved SBPH text */
 
   if ( CTX->tokenizer == DSZ_SBPH
-    && CTX->operating_mode != DSM_CLASSIFY 
-    && CTX->classification != DSR_NONE 
+    && CTX->operating_mode != DSM_CLASSIFY
+    && CTX->classification != DSR_NONE
     && CTX->flags & DSF_SIGNATURE)
   {
     char *y, *h, *b;
-    char *ptrptr;
+    char *ptrptr = NULL;
 
     y = strdup((const char *) CTX->signature->data);
     h = strtok_r(y, "\001", &ptrptr);
     b = strtok_r(NULL, "\001", &ptrptr);
     spam_result = _ds_operate (CTX, h, b);
+    free(y);
 
   /* Otherwise, operate on the input message */
- 
+
   } else {
     spam_result = _ds_operate (CTX, header->data, body->data);
   }
 
-  /* Force decision if a classification was specified */
+  /* Clean up */
+  buffer_destroy (header);
+  buffer_destroy (body);
 
-  if (CTX->classification != DSR_NONE && spam_result >= 0) 
-  {
+  /* _ds_operate() was unable to process message. Restore operating and training mode. */
+  if (spam_result != DSR_ISSPAM && spam_result != DSR_ISINNOCENT) {
+    LOG(LOG_WARNING, "received invalid result (!DSR_ISSPAM && !DSR_ISINNOCENT)"
+                     ": %d", spam_result);
+    retcode = EFAILURE;
+    goto restore_mode;
+  }
+
+  /* Force decision if a classification was specified */
+  if (CTX->classification != DSR_NONE) {
     if (CTX->classification == DSR_ISINNOCENT)
       spam_result = DSR_ISINNOCENT;
     else if (CTX->classification == DSR_ISSPAM)
       spam_result = DSR_ISSPAM;
   }
 
-  /* Apply results to context and clean up */
+  /* Apply results to context */
   CTX->result = spam_result;
   if (CTX->class[0] == 0) {
-    if (CTX->result == DSR_ISSPAM)
+    if (spam_result == DSR_ISSPAM)
       strcpy(CTX->class, LANG_CLASS_SPAM);
-    else if (CTX->result == DSR_ISINNOCENT)
+    else if (spam_result == DSR_ISINNOCENT)
       strcpy(CTX->class, LANG_CLASS_INNOCENT);
   }
-  buffer_destroy (header);
-  buffer_destroy (body);
+
+/* Restore operating mode and/or training mode */
+restore_mode:
 
   if (is_toe)
     CTX->operating_mode = DSM_PROCESS;
@@ -615,14 +621,7 @@ dspam_process (DSPAM_CTX * CTX, const char *message)
   }
 #endif
 
-  if (CTX->result == DSR_ISSPAM || CTX->result == DSR_ISINNOCENT) 
-    return 0;
-  else
-  {
-    LOG(LOG_WARNING, "received invalid result (! DSR_ISSPAM || DSR_INNOCENT) "
-                     ": %d", CTX->result);
-    return EUNKNOWN;
-  }
+  return retcode;
 }
 
 /*
@@ -700,7 +699,7 @@ dspam_getsource (
         }
         else
         {
-          char *ptrptr;
+          char *ptrptr = NULL;
           tok = strtok_r (ptr, "[", &ptrptr);
 
           if (tok != NULL)
@@ -760,7 +759,6 @@ int
 _ds_operate (DSPAM_CTX * CTX, char *headers, char *body)
 {
   int errcode = 0;
-  int i;
 
   /* Create our diction (lexical data in message) and patterns */
 
@@ -778,6 +776,7 @@ _ds_operate (DSPAM_CTX * CTX, char *headers, char *body)
   unsigned long long whitelist_token = 0;
   int do_whitelist = 0;
   int result;
+  unsigned int heap_sort_items = 0;
 
   if (CTX->algorithms & DSA_BURTON)
     heap_sort = ds_heap_create(BURTON_WINDOW_SIZE, HP_DELTA);
@@ -795,6 +794,12 @@ _ds_operate (DSPAM_CTX * CTX, char *headers, char *body)
        || ! (CTX->_sig_provided)) 
     && CTX->source != DSS_CORPUS)
   {
+    if (CTX->signature) {
+      if (CTX->signature->data)
+        free(CTX->signature->data);
+      free(CTX->signature);
+      CTX->signature = NULL;
+    }
     CTX->signature = calloc (1, sizeof (struct _ds_spam_signature));
     if (CTX->signature == NULL)
     {
@@ -822,7 +827,6 @@ _ds_operate (DSPAM_CTX * CTX, char *headers, char *body)
 
   if (!diction)
   {
-    ds_diction_destroy(diction);
     LOG (LOG_CRIT, ERR_MEM_ALLOC);
     errcode = EUNKNOWN;
     goto bail;
@@ -850,6 +854,7 @@ _ds_operate (DSPAM_CTX * CTX, char *headers, char *body)
       CTX->signature->length / sizeof (struct _ds_signature_token);
     struct _ds_signature_token t;
 
+    int i;
     for (i = 0; i < num_tokens; i++)
     {
       char x[128];
@@ -942,6 +947,9 @@ _ds_operate (DSPAM_CTX * CTX, char *headers, char *body)
   }
   ds_diction_close(ds_c);
 
+  /* Keep track of items in heap_sort. We need that info later on when freeing the signature */
+  heap_sort_items = heap_sort->items;
+
   /* Take the 15 most interesting tokens and generate a score */
 
   if (heap_sort->items == 0)
@@ -957,6 +965,12 @@ _ds_operate (DSPAM_CTX * CTX, char *headers, char *body)
     && CTX->flags & DSF_SIGNATURE 
     && (CTX->operating_mode != DSM_CLASSIFY || ! CTX->_sig_provided))
   {
+    if (CTX->signature) {
+      if (CTX->signature->data)
+        free(CTX->signature->data);
+      free(CTX->signature);
+      CTX->signature = NULL;
+    }
     CTX->signature = calloc (1, sizeof (struct _ds_spam_signature));
     if (CTX->signature == NULL)
     {
@@ -986,8 +1000,10 @@ _ds_operate (DSPAM_CTX * CTX, char *headers, char *body)
     if (CTX->flags & DSF_NOISE) {
       nobnr_result = _ds_calc_result(CTX, heap_nobnr, diction);
 
-      if (CTX->factors) 
+      if (CTX->factors) {
         _ds_factor_destroy(CTX->factors);
+        CTX->factors = NULL;
+      }
       CTX->result = x;
       CTX->probability = DSP_UNCALCULATED;
     }
@@ -1117,6 +1133,15 @@ bail:
 #endif
   ds_diction_destroy(diction);
   ds_diction_destroy(bnr_patterns);
+  if (CTX->signature != NULL) {
+    if (CTX->signature->data != NULL) {
+      free(CTX->signature->data);
+      CTX->signature->data = NULL;
+    }
+    if (CTX->signature != NULL && heap_sort_items > 0)
+      free (CTX->signature);
+    CTX->signature = NULL;
+  }
   return errcode;
 }
 
@@ -1154,7 +1179,7 @@ _ds_process_signature (DSPAM_CTX * CTX)
 
   LOGDEBUG ("processing signature.  length: %ld", CTX->signature->length);
 
-  CTX->result = DSS_NONE;
+  CTX->result = DSR_NONE;
 
   if (!(CTX->flags & DSF_UNLEARN)) 
     CTX->learned = 1;
@@ -1209,32 +1234,39 @@ _ds_process_signature (DSPAM_CTX * CTX)
       strcpy(CTX->class, LANG_CLASS_INNOCENT);
   }
 
-  LOGDEBUG ("reversing %d tokens", num_tokens);
-  for (i = 0; i < num_tokens; i++)
+  /* Don't retrain if no tokens where loaded from the signature */
+  if (num_tokens == 0)
   {
-    memcpy (&t,
-            (char *) CTX->signature->data +
-            (i * sizeof (struct _ds_signature_token)),
-            sizeof (struct _ds_signature_token));
-    ds_term = ds_diction_touch (diction, t.token, "-", 0);
-    if (ds_term)
-      ds_term->frequency = t.frequency;
-  }
-
-  if (_ds_getall_spamrecords (CTX, diction)) {
-    ds_diction_destroy(diction);
-    return EUNKNOWN;
-  }
-
-  ds_c = ds_diction_cursor(diction);
-  ds_term = ds_diction_next(ds_c);
-  while(ds_term)
-  {
-    /* INNOCENT */
-    if (CTX->classification == DSR_ISINNOCENT)
+    LOG (LOG_WARNING, "Skipping retraining for signature with %d tokens", num_tokens);
+    LOGDEBUG ("Skipping retraining for signature with %d tokens", num_tokens);
+  } else {
+    LOGDEBUG ("Reversing %d tokens", num_tokens);
+    for (i = 0; i < num_tokens; i++)
     {
-      if (CTX->flags & DSF_UNLEARN) {
-        if (CTX->classification == DSR_ISSPAM)
+      memcpy (&t,
+              (char *) CTX->signature->data +
+              (i * sizeof (struct _ds_signature_token)),
+              sizeof (struct _ds_signature_token));
+      ds_term = ds_diction_touch (diction, t.token, "-", 0);
+      if (ds_term)
+      {
+        ds_term->frequency = t.frequency;
+      }
+    }
+
+    if (_ds_getall_spamrecords (CTX, diction)) {
+      ds_diction_destroy(diction);
+      return EUNKNOWN;
+    }
+
+    ds_c = ds_diction_cursor(diction);
+    ds_term = ds_diction_next(ds_c);
+    while(ds_term)
+    {
+      /* INNOCENT */
+      if (CTX->classification == DSR_ISINNOCENT)
+      {
+        if (CTX->flags & DSF_UNLEARN)
         {
           if (occurrence)
           {
@@ -1244,19 +1276,47 @@ _ds_process_signature (DSPAM_CTX * CTX)
           } else {
             ds_term->s.innocent_hits -= (ds_term->s.innocent_hits>0) ? 1:0;
           }
-        }
-
-      } else {
-        if (occurrence)
-        {
-          ds_term->s.innocent_hits += ds_term->frequency;
         } else {
-          ds_term->s.innocent_hits++;
-        }
+          if (CTX->source == DSS_ERROR          && 
+              CTX->training_mode != DST_NOTRAIN && 
+              CTX->training_mode != DST_TOE)
+          {
+            if (occurrence)
+            {
+              ds_term->s.spam_hits -= ds_term->frequency;
+              if (ds_term->s.spam_hits < 0)
+                ds_term->s.spam_hits = 0;
+            } else {
+              ds_term->s.spam_hits -= (ds_term->s.spam_hits>0) ? 1:0;
+            }
+          }
 
-        if (CTX->source == DSS_ERROR          && 
-            CTX->training_mode != DST_NOTRAIN && 
-            CTX->training_mode != DST_TOE)
+          if (CTX->source == DSS_INOCULATION)
+          {
+            if (ds_term->s.spam_hits < 2 && ds_term->s.innocent_hits < 5)
+            {
+              ds_term->s.innocent_hits += 5;
+            }
+            else
+            {
+              ds_term->s.innocent_hits += 2;
+            }
+          } else /* ERROR or CORPUS */
+          {
+            if (occurrence)
+            {
+              ds_term->s.innocent_hits += ds_term->frequency;
+            } else {
+              ds_term->s.innocent_hits++;
+            }
+          }
+        }
+      }
+
+      /* SPAM */
+      else if (CTX->classification == DSR_ISSPAM)
+      {
+        if (CTX->flags & DSF_UNLEARN)
         {
           if (occurrence)
           {
@@ -1266,71 +1326,53 @@ _ds_process_signature (DSPAM_CTX * CTX)
           } else {
             ds_term->s.spam_hits -= (ds_term->s.spam_hits>0) ? 1:0;
           }
-        }
-      }
-    }
-
-
-    /* SPAM */
-    else if (CTX->classification == DSR_ISSPAM)
-    {
-      if (CTX->flags & DSF_UNLEARN) {
-        if (CTX->classification == DSR_ISSPAM)
-        {
-          if (occurrence)
+        } else {
+          if (CTX->source == DSS_ERROR          && 
+              CTX->training_mode != DST_NOTRAIN && 
+              CTX->training_mode != DST_TOE)
           {
-            ds_term->s.spam_hits -= ds_term->frequency;
-            if (ds_term->s.spam_hits < 0)
-              ds_term->s.spam_hits = 0;
-          } else {
-            ds_term->s.spam_hits -= (ds_term->s.innocent_hits>0) ? 1:0;
-          }
-        }
-
-      } else {
-       if (CTX->source == DSS_ERROR          && 
-           CTX->training_mode != DST_NOTRAIN && 
-           CTX->training_mode != DST_TOE)
-       {
-          if (occurrence)
-          {
-            ds_term->s.innocent_hits -= ds_term->frequency;
-            if (ds_term->s.innocent_hits < 0)
-              ds_term->s.innocent_hits = 0;
-          } else {
-            ds_term->s.innocent_hits -= (ds_term->s.innocent_hits>0) ? 1:0;
+            if (occurrence)
+            {
+              ds_term->s.innocent_hits -= ds_term->frequency;
+              if (ds_term->s.innocent_hits < 0)
+                ds_term->s.innocent_hits = 0;
+            } else {
+              ds_term->s.innocent_hits -= (ds_term->s.innocent_hits>0) ? 1:0;
+            }
           }
 
-       }
-
-        if (CTX->source == DSS_INOCULATION)
-        {
-          if (ds_term->s.innocent_hits < 2 && ds_term->s.spam_hits < 5)
-            ds_term->s.spam_hits += 5;
-          else
-            ds_term->s.spam_hits += 2;
-        } else /* ERROR or CORPUS */
-        {
+          if (CTX->source == DSS_INOCULATION)
+          {
+            if (ds_term->s.innocent_hits < 2 && ds_term->s.spam_hits < 5)
+            {
+              ds_term->s.spam_hits += 5;
+            }
+            else
+            {
+              ds_term->s.spam_hits += 2;
+            }
+          } else /* ERROR or CORPUS */
+          {
             if (occurrence)
             {
               ds_term->s.spam_hits += ds_term->frequency;
             } else {
               ds_term->s.spam_hits++;
             }
-
+          }
         }
       }
+
+      ds_term->s.status |= TST_DIRTY;
+      ds_term = ds_diction_next(ds_c);
     }
+    ds_diction_close(ds_c);
 
-    ds_term->s.status |= TST_DIRTY;
-    ds_term = ds_diction_next(ds_c);
-  }
-  ds_diction_close(ds_c);
-
-  if (CTX->training_mode != DST_NOTRAIN) {
-    if (_ds_setall_spamrecords (CTX, diction)) {
-      ds_diction_destroy(diction);
-      return EUNKNOWN;
+    if (CTX->training_mode != DST_NOTRAIN) {
+      if (_ds_setall_spamrecords (CTX, diction)) {
+        ds_diction_destroy(diction);
+        return EUNKNOWN;
+      }
     }
   }
 
@@ -1338,11 +1380,13 @@ _ds_process_signature (DSPAM_CTX * CTX)
   {
     CTX->probability = 1.0;
     CTX->result = DSR_ISSPAM;
+    LOGDEBUG ("Message classification/result: SPAM");
   }
   else
   {
     CTX->probability = 0.0;
     CTX->result = DSR_ISINNOCENT;
+    LOGDEBUG ("Message classification/result: INNOCENT");
   }
 
   ds_diction_destroy(diction);
@@ -1375,7 +1419,7 @@ _ds_calc_stat (
   struct _ds_spam_stat *bnr_tot)
 {
   int min_hits, sed_hits = 0;
-  long ti, ts;
+  unsigned long ti, ts;
 
   if (token_type == DTT_BNR) {
     min_hits = 25; /* Bayesian Noise Reduction patterns */
@@ -1384,8 +1428,8 @@ _ds_calc_stat (
     min_hits = 5; /* "Standard" token threshold */
   }
 
-  /*  Statistical Sedation: Adjust hapaxial threshold to compensate for a 
-   *  spam corpus imbalance 
+  /*  Statistical Sedation: Adjust hapaxial threshold to compensate for a
+   *  spam corpus imbalance
    */
 
   ti = CTX->totals.innocent_learned + CTX->totals.innocent_classified;
@@ -1407,19 +1451,19 @@ _ds_calc_stat (
     min_hits = 5;
   }
 
-  if (token_type != DTT_DEFAULT || sed_hits > min_hits) 
+  if (token_type != DTT_DEFAULT || sed_hits > min_hits)
     min_hits = sed_hits;
 
   /*  TUM mode training only records up to 20 hits so we need to make sure we
    *  don't require more than that.
    */
 
-  if (CTX->training_mode == DST_TUM && min_hits > 20) 
+  if (CTX->training_mode == DST_TUM && min_hits > 20)
     min_hits = 20;
 
   if (CTX->classification == DSR_ISSPAM)
     s->probability = .7;
-  else 
+  else
     s->probability = (CTX->algorithms & DSP_MARKOV) ? .5 : .4;
 
   /* Markovian Weighting */
@@ -1427,7 +1471,7 @@ _ds_calc_stat (
   if (CTX->algorithms & DSP_MARKOV) {
     unsigned int weight;
     long num, den;
- 
+
     /*  some utilities don't provide the token name, and so we can't compute
      *  a probability. just return something neutral.
      */
@@ -1436,20 +1480,40 @@ _ds_calc_stat (
       return 0;
     }
 
+    /* return neutral probability for BNR patterns */
+    if (token_type == DTT_BNR || term->type == 'B' || !strncmp(term->name, "bnr.", 4)) {
+      s->probability = .5;
+      return 0;
+    }
+
+    /* return neutral probability for frequency tokens */
+    if (!strncmp(term->name, "E: ", 3)) {
+      s->probability = .5;
+      return 0;
+    }
+
+    /* return neutral probability for "From" tokens (used for when whitelisting) */
+    if (!strncmp(term->name, "From*", 5)) {
+      s->probability = .5;
+      return 0;
+    }
+
+    /* return neutral probability for control tokens */
+    if (!strncmp(term->name, "$$CONTROL$$", 11)) {
+      s->probability = .5;
+      return 0;
+    }
+
     weight = _ds_compute_weight(term->name);
 
     if (CTX->flags & DSF_BIAS) {
       num = weight * (s->spam_hits - (s->innocent_hits*2));
       den = C1 * (s->spam_hits + (s->innocent_hits*2) + C2) * 256;
+      s->probability = 0.49 + ((double) num / (double) den);
     } else {
       num = (s->spam_hits - s->innocent_hits) * weight;
       den = C1 * (s->spam_hits + s->innocent_hits + C2) * 256;
-    }
-
-    if (CTX->flags & DSF_BIAS) {
-      s->probability = 0.49 + ((double) num / (double) den);
-    } else {
-      s->probability = 0.5 + ((double) num / (double) den); 
+      s->probability = 0.5 + ((double) num / (double) den);
     }
 
   /* Graham and Robinson Start Here */
@@ -1524,7 +1588,7 @@ _ds_calc_stat (
 
   if (token_type != DTT_BNR && CTX->algorithms & DSP_ROBINSON)
   {
-    int n = s->spam_hits + s->innocent_hits;
+    unsigned long n = s->spam_hits + s->innocent_hits;
     double fw = ((CHI_S * CHI_X) + (n * s->probability))/(CHI_S + n);
     s->probability = fw;
   }
@@ -1539,9 +1603,9 @@ _ds_calc_stat (
  *   Perform statistical combination of the token index
  *
  *    Passed in an index of tokens, this function is responsible for choosing
- *    and combining  the most  relevant characteristics  (based on the  algo-
- *    rithms  configured)  and calculating  libdspam's  decision  about  the 
- *    provided message sample.
+ *    and combining the most relevant characteristics (based on the algorithms
+ *    configured) and calculating libdspam's decision about the provided
+ *    message sample.
  */
 
 int
@@ -1559,14 +1623,14 @@ _ds_calc_result(DSPAM_CTX *CTX, ds_heap_t heap_sort, ds_diction_t diction)
   struct nt *factor_nbayes = nt_create(NT_PTR);
 
   /* Graham-Bayesian */
-  float bay_top = 0.0; 
+  float bay_top = 0.0;
   float bay_bot = 0.0;
   float bay_result = -1;
   long bay_used = 0;            /* Total tokens used in bayes */
   struct nt *factor_bayes = nt_create(NT_PTR);
 
   /* Burton-Bayesian */
-  double abay_top = 0.0; 
+  double abay_top = 0.0;
   double abay_bot = 0.0;
   double abay_result = -1;
   long abay_used = 0;           /* Total tokens used in altbayes */
@@ -1579,13 +1643,13 @@ _ds_calc_result(DSPAM_CTX *CTX, ds_heap_t heap_sort, ds_diction_t diction)
   double p = 0.0, q = 0.0, s = 0.0;     /* Robinson PQS Calculations */
   long rob_used = 0;            	/* Total tokens used in Robinson's GM */
   struct nt *factor_rob = nt_create(NT_PTR);
-                                                                                
+
   /* Fisher-Robinson's Chi-Square */
   float chi_result = -1;
   long chi_used  = 0, chi_sx = 0, chi_hx = 0;
   double chi_s = 1.0, chi_h = 1.0;
   struct nt *factor_chi = nt_create(NT_PTR);
-  int i;
+  unsigned int i;
 
   /* Invert the heap */
   node_heap = heap_sort->root;
@@ -1594,8 +1658,6 @@ _ds_calc_result(DSPAM_CTX *CTX, ds_heap_t heap_sort, ds_diction_t diction)
     node_heap = node_heap->next;
   }
 
-  node_heap = heap_sort->root;
-
   /* BEGIN Combine Token Values */
   for(i=0;i<heap_sort->items;i++)
   {
@@ -1605,7 +1667,11 @@ _ds_calc_result(DSPAM_CTX *CTX, ds_heap_t heap_sort, ds_diction_t diction)
     node_heap = heap_list[i];
     ds_term = ds_diction_find(diction, node_heap->token);
 
-    if (!ds_term) 
+    if (!ds_term)
+      continue;
+
+    /* Skip BNR patterns */
+    if (ds_term->type == 'B')
       continue;
 
     token_name = ds_term->name;
@@ -1613,10 +1679,6 @@ _ds_calc_result(DSPAM_CTX *CTX, ds_heap_t heap_sort, ds_diction_t diction)
     if (ds_diction_getstat(diction, node_heap->token, &stat) || !token_name)
       continue;
 
-    /* Skip BNR patterns */
-    if (ds_term->type == 'B')
-      continue;
-
     /* Set the probability if we've provided a classification */
     if (CTX->classification == DSR_ISSPAM)
       stat.probability = 1.00;
@@ -1788,9 +1850,9 @@ _ds_calc_result(DSPAM_CTX *CTX, ds_heap_t heap_sort, ds_diction_t diction)
           LOGDEBUG ("[naive] [%2.6f] %s (%dfrq, %lds, %ldi)",
                     ds_term->s.probability, ds_term->name, ds_term->frequency,
                     ds_term->s.spam_hits, ds_term->s.innocent_hits);
-  
+
         _ds_factor(factor_nbayes, ds_term->name, stat.probability);
-  
+
         if (nbay_used == 0)
         {
           nbay_top = stat.probability;
@@ -1801,7 +1863,7 @@ _ds_calc_result(DSPAM_CTX *CTX, ds_heap_t heap_sort, ds_diction_t diction)
           nbay_top *= stat.probability;
           nbay_bot *= (1 - stat.probability);
         }
-  
+
         nbay_used++;
       }
 
@@ -1821,9 +1883,8 @@ _ds_calc_result(DSPAM_CTX *CTX, ds_heap_t heap_sort, ds_diction_t diction)
         }
 
         if (fabs(0.5-fw)>CHI_EXCR) {
-          int iter = _ds_compute_complexity(ds_term->name);
+          int iter = 1;
 
-          iter = 1;
           while(iter>0) {
             iter --;
 
@@ -1839,7 +1900,7 @@ _ds_calc_result(DSPAM_CTX *CTX, ds_heap_t heap_sort, ds_diction_t diction)
 #endif
 
             _ds_factor(factor_chi, ds_term->name, ds_term->s.probability);
-  
+
             chi_used++;
             chi_s *= (1.0 - fw);
             chi_h *= fw;
@@ -1849,7 +1910,7 @@ _ds_calc_result(DSPAM_CTX *CTX, ds_heap_t heap_sort, ds_diction_t diction)
             }
             if (chi_h < 1e-200) {
               chi_h = frexp(chi_h, &exp);
-              chi_hx += exp; 
+              chi_hx += exp;
             }
           }
         }
@@ -1896,7 +1957,7 @@ CHI_NEXT:
 
   rob_result = s;
 
-  if (CTX->algorithms & DSA_ROBINSON) { 
+  if (CTX->algorithms & DSA_ROBINSON) {
     LOGDEBUG("Robinson's Geometric Confidence: %f (Spamminess: %f, "
       "Non-Spamminess: %f, Samples: %ld)", rob_result, p, q, rob_used);
   }
@@ -1912,9 +1973,9 @@ CHI_NEXT:
       chi_result = ((chi_s-chi_h)+1.0) / 2.0;
     } else {
       chi_result = (float)(CHI_CUTOFF-0.1);
-    }  
+    }
 
-    LOGDEBUG("Chi-Square Confidence: %f", chi_result); 
+    LOGDEBUG("Chi-Square Confidence: %f", chi_result);
   }
 
 /* END Calculate Individual Probabilities */
@@ -1932,7 +1993,7 @@ CHI_NEXT:
 
     if (CTX->algorithms & DSA_NAIVE) {
       factor = factor_nbayes;
-      if ((CTX->algorithms & DSP_MARKOV && nbay_result > 0.5000) ||
+      if (((CTX->algorithms & DSP_MARKOV) && nbay_result > 0.5000) ||
           (!(CTX->algorithms & DSP_MARKOV) && nbay_result >= 0.9))
       {
         CTX->result = DSR_ISSPAM;
@@ -1944,7 +2005,7 @@ CHI_NEXT:
 
     if (CTX->algorithms & DSA_GRAHAM) {
       factor = factor_bayes;
-      if ((CTX->algorithms & DSP_MARKOV && bay_result > 0.5000) ||
+      if (((CTX->algorithms & DSP_MARKOV) && bay_result > 0.5000) ||
           (!(CTX->algorithms & DSP_MARKOV) && bay_result >= 0.9))
       {
         CTX->result = DSR_ISSPAM;
@@ -1956,7 +2017,7 @@ CHI_NEXT:
 
     if (CTX->algorithms & DSA_BURTON) {
       factor = factor_altbayes;
-      if ((CTX->algorithms & DSP_MARKOV && abay_result > 0.5000) ||
+      if (((CTX->algorithms & DSP_MARKOV) && abay_result > 0.5000) ||
           (!(CTX->algorithms & DSP_MARKOV) && abay_result >= 0.9))
       {
         CTX->result = DSR_ISSPAM;
@@ -1970,7 +2031,7 @@ CHI_NEXT:
 
     if (CTX->algorithms & DSA_ROBINSON) {
       factor = factor_rob;
-      if ((CTX->algorithms & DSP_MARKOV && rob_result > 0.5000) ||
+      if (((CTX->algorithms & DSP_MARKOV) && rob_result > 0.5000) ||
           (!(CTX->algorithms & DSP_MARKOV) && rob_result >= ROB_CUTOFF))
       {
         CTX->result = DSR_ISSPAM;
@@ -1985,7 +2046,7 @@ CHI_NEXT:
 
     if (CTX->algorithms & DSA_CHI_SQUARE) {
      factor = factor_chi;
-     if ((CTX->algorithms & DSP_MARKOV && chi_result > 0.5000) ||
+     if (((CTX->algorithms & DSP_MARKOV) && chi_result > 0.5000) ||
          (!(CTX->algorithms & DSP_MARKOV) && chi_result >= CHI_CUTOFF))
      {
        CTX->result = DSR_ISSPAM;
@@ -2044,7 +2105,7 @@ CHI_NEXT:
     {
       LOGDEBUG ("MISS: Burton Bayesian");
     }
-                                                                                
+
     if (rob_result >= ROB_CUTOFF && bay_result < 0.9)
     {
       LOGDEBUG ("CATCH: Robinson's");
@@ -2053,7 +2114,7 @@ CHI_NEXT:
     {
       LOGDEBUG ("MISS: Robinson's");
     }
-                                                                                
+
     if (chi_result >= CHI_CUTOFF && bay_result < 0.9)
     {
       LOGDEBUG("CATCH: Chi-Square");
@@ -2136,32 +2197,27 @@ void _ds_factor_destroy(struct nt *factors) {
   nt_destroy(factors);
 
   return;
-} 
+}
 
 int libdspam_init(const char *driver) {
 
 #ifndef STATIC_DRIVER
-  if (driver) {
+  if (driver == NULL) {
+      LOG(LOG_CRIT, "dlopen() failed: Can not load NULL driver");
+      return EFAILURE;
+  } else if (driver) {
     if ((_drv_handle = dlopen(driver, RTLD_NOW))==NULL) {
-      LOG(LOG_CRIT, "dlopen() failed: %s: %s", driver, dlerror()); 
+      LOG(LOG_CRIT, "dlopen() failed: %s: %s", driver, dlerror());
       return EFAILURE;
     }
   }
 #endif
 
-#ifdef NCORE
-  _ncsetup();
-#endif
-
   return 0;
 }
 
 int libdspam_shutdown(void) {
 
-#ifdef NCORE
-  _nccleanup();
-#endif
-
 #ifndef STATIC_DRIVER
   if (_drv_handle) {
     int r;
@@ -2171,7 +2227,7 @@ int libdspam_shutdown(void) {
     }
   }
 #endif
-   
+
   return 0;
 }
 
@@ -2231,9 +2287,6 @@ ds_diction_t _ds_apply_bnr (DSPAM_CTX *CTX, ds_diction_t diction) {
   struct _ds_spam_stat bnr_tot;
   unsigned long long crc;
   BNR_CTX *BTX_S, *BTX_C;
-#ifdef LIBBNR_DEBUG
-  float snr;
-#endif
   struct nt_node *node_nt;
   struct nt_c c_nt;
   ds_term_t ds_term, ds_touch;
@@ -2252,6 +2305,7 @@ ds_diction_t _ds_apply_bnr (DSPAM_CTX *CTX, ds_diction_t diction) {
     LOGDEBUG("bnr_init() failed");
     bnr_destroy(BTX_S);
     bnr_destroy(BTX_C);
+    ds_diction_destroy(bnr_patterns);
     return NULL;
   }
 
@@ -2271,6 +2325,7 @@ ds_diction_t _ds_apply_bnr (DSPAM_CTX *CTX, ds_diction_t diction) {
   LOGDEBUG("Loading %ld BNR patterns", bnr_patterns->items);
   if (_ds_getall_spamrecords (CTX, bnr_patterns)) {
     LOGDEBUG ("_ds_getall_spamrecords() failed");
+    ds_diction_destroy(bnr_patterns);
     return NULL;
   }
 
@@ -2341,6 +2396,7 @@ ds_diction_t _ds_apply_bnr (DSPAM_CTX *CTX, ds_diction_t diction) {
     }
 
 #ifdef LIBBNR_DEBUG
+    float snr;
     if (BTX_S->stream->items + BTX_C->stream->items +
         BTX_S->eliminations  + BTX_C->eliminations > 0)
     {
