@@ -1,22 +1,21 @@
-/* $Id: libdspam_objects.h,v 1.16 2006/01/24 14:39:38 jonz Exp $ */
+/* $Id: libdspam_objects.h,v 1.27 2011/07/11 21:29:57 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 DEEP LOGIC INC.
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -34,6 +33,7 @@
 
 #if ((defined(__sun__) && defined(__svr4__)) || (defined(__sun) && defined(__SUNPRO_C))) && !defined(u_int32_t) && !defined(__BIT_TYPES_DEFINED__)
 #define __BIT_TYPES_DEFINED__
+typedef unsigned long long u_int64_t;
 typedef unsigned int u_int32_t;
 typedef unsigned short u_int16_t;
 typedef unsigned char u_int8_t;
@@ -53,7 +53,7 @@ extern void *_drv_handle; /* Handle to storage driver library */
  *  a message.  An array of these are returned to the calling  application to
  *  explain libdspam's final classification decision.
  */
-                                                                                
+
 struct dspam_factor {
   char *token_name;
   float value;
@@ -120,6 +120,7 @@ typedef struct _ds_spam_stat
   long spam_hits;
   long innocent_hits;
   char status;
+  unsigned long offset;
 } *ds_spam_stat_t;
 
 /*
@@ -138,7 +139,7 @@ typedef struct _ds_spam_stat
 struct _ds_spam_signature
 {
   void *data;
-  long length;
+  unsigned long length;
 };
 
 /*
@@ -258,24 +259,31 @@ struct _ds_config
  *  flags (input)
  *    Applies different fine-tuning behavior to the context:
  *
- *	DSF_CHAINED		Use chained tokenizer
- *      DSF_SBPH                Use SBPH tokenizer
  *	DSF_NOISE		Apply Bayesian Noise Reduction logic
  *	DSF_SIGNATURE		Signature is provided/requested
  *      DSF_WHITELIST		Use automatic whitelisting logic
  *      DSF_MERGED		Merge user/group data in memory
  *      DSF_UNLEARN		Unlearn the message
+ *      DSF_BIAS		Assign processor bias to unknown tokens
+ *
+ *  tokenizer (input)
+ *    Specifies which tokenizer to use
+ *
+ *      DSZ_WORD		Use WORD (uniGram) tokenizer
+ *      DSZ_CHAIN		Use CHAIN (biGram) tokenizer
+ *      DSZ_SBPH		Use SBPH (Sparse Binary Polynomial Hashing) tokenizer
+ *      DSZ_OSB			Use OSB (Orthogonal Sparse biGram) tokenizer
  *
  *  algorithms (input)
  *    Optional API to override the default algorithms. This value is set
  *    with the default compiled values whenever dspam_create() is called.
  *
  *	DSA_GRAHAM		Graham-Bayesian
- *	DSA_BURTON		Burton-Bayesian    
+ *	DSA_BURTON		Burton-Bayesian
  *	DSA_ROBINSON		Robinson's Geometric Mean Test
  *	DSA_CHI_SQUARE		Fisher-Robinson's Chi-Square
  *      DSA_NAIVE		Naive-Bayesian
- * 
+ *
  *    P-Value Computations:
  *
  *      DSP_ROBINSON		Robinson's Technique
@@ -303,6 +311,7 @@ typedef struct
   int		classification;  /* DSR_ */
   int		source;		 /* DSS_ */
   int		learned;	 /* Did we actually learn something? */
+  int           tokenizer;       /* DSZ_ */
   u_int32_t	flags;
   u_int32_t	algorithms;
 
@@ -322,15 +331,20 @@ typedef struct
 
 /* Processing Flags */
 
-#define DSF_CHAINED		0x01
 #define DSF_SIGNATURE		0x02
 #define DSF_BIAS		0x04
 #define DSF_NOISE		0x08
 #define DSF_WHITELIST		0x10
 #define DSF_MERGED		0x20
-#define DSF_SBPH		0x40
 #define DSF_UNLEARN		0x80
 
+/* Tokenizers */
+
+#define DSZ_WORD		0x01
+#define DSZ_CHAIN		0x02
+#define DSZ_SBPH		0x03
+#define DSZ_OSB			0x04
+
 /* Algorithms */
 
 #define DSA_GRAHAM		0x01
@@ -349,13 +363,12 @@ typedef struct
 #define DSM_CLASSIFY		0x02
 #define DSM_NONE		0xFF
 
-/* Training Modes */ 
+/* Training Modes */
 
 #define DST_TEFT		0x00
 #define DST_TOE			0x01
 #define DST_TUM			0x02
 #define DST_NOTRAIN		0xFE
-#define DST_DEFAULT		0xFF
 
 /* Classification Results */
 
@@ -380,4 +393,6 @@ typedef struct
 
 #define DSP_UNCALCULATED	-1
 
+#define BURTON_WINDOW_SIZE	27
+
 #endif /* _LIBDSPAM_OBJECTS */
