@@ -1,22 +1,21 @@
-/* $Id: list.c,v 1.7 2006/05/13 01:12:59 jonz Exp $ */
+/* $Id: list.c,v 1.76 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -33,10 +32,10 @@ static struct bnr_list_node *
 bnr_list_node_create (void *data)
 {
   struct bnr_list_node *node;
-  if ((node = (struct bnr_list_node *) malloc (sizeof (struct bnr_list_node))) == 0)
+  if ((node = (struct bnr_list_node *) malloc (sizeof (struct bnr_list_node))) == NULL)
   {
-    perror("memory allocation error: list_node_create() failed");
-    exit (1);
+    perror("list_node_create: memory allocation error");
+    return NULL;
   }
   node->ptr = data;
   node->next = (struct bnr_list_node *) NULL;
@@ -51,7 +50,7 @@ bnr_list_create (int nodetype)
   struct bnr_list *list = (struct bnr_list *) malloc (sizeof (struct bnr_list));
   if (list == NULL)
   {
-    perror("memory allocation error: list_create() failed");
+    perror("bnr_list_create: memory allocation error");
     return NULL;
   }
   list->first = (struct bnr_list_node *) NULL;
@@ -82,7 +81,7 @@ bnr_list_destroy (struct bnr_list *list)
     cur = next;
   }
   free (list);
-  list = (struct bnr_list *) NULL;
+  /* list = (struct bnr_list *) NULL; */
 }
 
 /* list_insert adds an item to the nodetree */
@@ -113,7 +112,7 @@ bnr_list_insert (struct bnr_list *list, void *data, float value)
     vptr = malloc (size);
     if (vptr == NULL)
     {
-      perror("memory allocation error: list_insert() failed");
+      perror("bnr_list_insert: memory allocation error");
       return NULL;
     }
     strcpy (vptr, data);
@@ -126,6 +125,8 @@ bnr_list_insert (struct bnr_list *list, void *data, float value)
   if (prev)
   {
     node = bnr_list_node_create (vptr);
+    if (node == NULL)
+      return NULL;
     node->value = value;
     node->eliminated = 0;
     prev->next = node;
@@ -135,6 +136,8 @@ bnr_list_insert (struct bnr_list *list, void *data, float value)
   else
   {
     node = bnr_list_node_create (vptr);
+    if (node == NULL)
+      return NULL;
     node->value = value;
     node->eliminated = 0;
     list->first = node;
