@@ -1,22 +1,21 @@
-/* $Id: pgsql_drv.h,v 1.11 2006/05/13 01:12:59 jonz Exp $ */
+/* $Id: pgsql_drv.h,v 1.16 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -33,6 +32,8 @@ struct _pgsql_drv_storage
 {
   PGconn *dbh;                   /* database connection */
   int pg_major_ver;              /* database major version */
+  int pg_minor_ver;              /* database minor version */
+  int pg_micro_ver;              /* database micro version */
   int pg_token_type;             /* type of token  */
 
   struct _ds_spam_totals control_totals;        /* totals at storage init */
@@ -64,7 +65,7 @@ void	_pgsql_drv_query_error		(const char *error, const char *query);
 int	_pgsql_drv_token_type		(struct _pgsql_drv_storage *s, PGresult *result, int column);
 char	*_pgsql_drv_token_write		(int type, unsigned long long token, char *buffer, size_t bufsz);
 unsigned long long _pgsql_drv_token_read(int type, char *str);
-int	_pgsql_drv_get_dbversion	(struct _pgsql_drv_storage *s);
+int	_pgsql_drv_get_dbversion	(struct _pgsql_drv_storage *s, unsigned int range);
 PGconn *_pgsql_drv_connect		(DSPAM_CTX *CTX);
 struct passwd *_pgsql_drv_getpwnam      (DSPAM_CTX * CTX, const char *name);
 struct passwd *_pgsql_drv_getpwuid      (DSPAM_CTX * CTX, uid_t uid);
@@ -75,6 +76,10 @@ DSPAM_CTX *_pgsql_drv_init_tools( const char *home, config_t config,
 struct passwd *_pgsql_drv_setpwnam	(DSPAM_CTX * CTX, const char *name);
 #endif
 
+#ifdef EXT_LOOKUP
+int verified_user;
+#endif
+
 #ifdef PREFERENCES_EXTENSION
 int _pgsql_drv_set_attributes(DSPAM_CTX *CTX, config_t config);
 #endif
