@@ -1,22 +1,21 @@
-/* $Id: pref.c,v 1.32 2006/05/13 01:12:59 jonz Exp $ */
+/* $Id: pref.c,v 1.38 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -55,7 +54,7 @@
 
 agent_pref_t _ds_pref_aggregate(agent_pref_t STX, agent_pref_t UTX) {
   agent_pref_t PTX = calloc(1, PREF_MAX*sizeof(agent_attrib_t ));
-  int i, j, size = 0;
+  int i, size = 0;
 
   if (STX) {
     for(i=0;STX[i];i++) {
@@ -71,6 +70,8 @@ agent_pref_t _ds_pref_aggregate(agent_pref_t STX, agent_pref_t UTX) {
       if (_ds_match_attribute(agent_config, "AllowOverride", UTX[i]->attribute))
       {
         int found = 0;
+        int j;
+
         for(j=0;PTX[j];j++) {
           if (!strcasecmp(PTX[j]->attribute, UTX[i]->attribute)) {
             found = 1;
@@ -159,7 +160,7 @@ agent_attrib_t _ds_pref_new(const char *attribute, const char *value) {
 
 agent_pref_t _ds_ff_pref_load(
   config_t config,
-  const char *user, 
+  const char *user,
   const char *home,
   void *ignore)
 {
@@ -167,8 +168,10 @@ agent_pref_t _ds_ff_pref_load(
   agent_pref_t PTX = malloc(sizeof(agent_attrib_t )*PREF_MAX);
   char buff[258];
   FILE *file;
-  char *p, *q, *bufptr;
+  char *p, *q;
   int i = 0;
+  config = config; /* Keep compiler happy */
+  ignore = ignore; /* Keep compiler happy */
 
   if (PTX == NULL) {
     LOG(LOG_CRIT, ERR_MEM_ALLOC);
@@ -182,9 +185,9 @@ agent_pref_t _ds_ff_pref_load(
     _ds_userdir_path (filename, home, user, "prefs");
   }
   file = fopen(filename, "r");
- 
+
   /* Apply default preferences from dspam.conf */
-                                                                                
+
   if (file != NULL) {
     char *ptrptr;
     while(i<(PREF_MAX-1) && fgets(buff, sizeof(buff), file)!=NULL) {
@@ -192,8 +195,6 @@ agent_pref_t _ds_ff_pref_load(
         continue;
       chomp(buff);
 
-      bufptr = buff;
-
       p = strtok_r(buff, "=", &ptrptr);
 
       if (p == NULL)
@@ -305,6 +306,8 @@ int _ds_ff_pref_set (
 {
   char filename[MAX_FILENAME_LENGTH];
   FILE *out_file;
+  config = config; /* Keep compiler happy */
+  ignore = ignore; /* Keep compiler happy */
 
   if (username == NULL) {
     snprintf(filename, MAX_FILENAME_LENGTH, "%s/default.prefs", home);
@@ -332,6 +335,8 @@ int _ds_ff_pref_del (
   char filename[MAX_FILENAME_LENGTH];
   FILE *out_file;
   int nlines; 
+  config = config; /* Keep compiler happy */
+  ignore = ignore; /* Keep compiler happy */
 
   if (username == NULL) {
     snprintf(filename, MAX_FILENAME_LENGTH, "%s/default.prefs", home);
