@@ -1,22 +1,21 @@
-/* $Id: sqlite_drv.c,v 1.29 2006/05/17 11:53:23 jonz Exp $ */
+/* $Id: sqlite_drv.c,v 1.34 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -648,8 +647,10 @@ _ds_init_storage (DSPAM_CTX * CTX, void *dbh)
                                                                                 
   if (s->dbh == NULL)
   {
+    free(s);
     LOGDEBUG
-      ("_ds_init_storage: sqlite_open: unable to initialize database: %s", err);    return EUNKNOWN;
+      ("_ds_init_storage: sqlite_open: unable to initialize database: %s", err);
+    return EUNKNOWN;
   }
 
   /* Commit timeout of 20 minutes */
@@ -1091,7 +1092,6 @@ _ds_get_nextuser (DSPAM_CTX * CTX)
         s->dir_handles->first = NULL;
       free (node_nt);
       s->dir_handles->items--;
-      prev = node_nt;
       break;
     }
     prev = node_nt;
@@ -1223,6 +1223,7 @@ _ds_get_nextsignature (DSPAM_CTX * CTX)
   length = strtol(row[3], NULL, 0);
   if (length == 0)
   {
+    free(st);
     return _ds_get_nextsignature(CTX);
   }
 
