@@ -1,22 +1,21 @@
-/* $Id: tokenizer.c,v 1.23 2006/05/27 20:17:27 jonz Exp $ */
+/* $Id: tokenizer.c,v 1.301 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -62,16 +61,6 @@
 #include "util.h"
 #include "libdspam.h"
 #include "language.h"
-#ifdef NCORE
-#include <ncore/ncore.h>
-#include <ncore/types.h>
-#include "ncore_adp.h"
-#endif
-
-#ifdef NCORE
-extern nc_dev_t		g_ncDevice;
-extern NC_STREAM_CTX	g_ncDelimiters;
-#endif
 
 /*
  * _ds_tokenize() - tokenize the message
@@ -104,16 +93,13 @@ _ds_tokenize (DSPAM_CTX * CTX, char *headers, char *body, ds_diction_t diction)
 }
 
 int _ds_tokenize_ngram(
-  DSPAM_CTX *CTX, 
-  char *headers, 
-  char *body, 
+  DSPAM_CTX *CTX,
+  char *headers,
+  char *body,
   ds_diction_t diction)
 {
   char *token;				/* current token */
   char *previous_token = NULL;		/* used for bigrams (chained tokens) */
-#ifdef NCORE
-  nc_strtok_t NTX;
-#endif
   char *line = NULL;			/* header broken up into lines */
   char *ptrptr;
   char heading[128];			/* current heading */
@@ -132,7 +118,7 @@ int _ds_tokenize_ngram(
   }
 
   /*
-   * Header Tokenization 
+   * Header Tokenization
    */
  
   header = nt_create (NT_CHAR);
@@ -230,47 +216,44 @@ int _ds_tokenize_ngram(
   LOGDEBUG("parsing message body");
 #endif
 
-#ifdef NCORE
-  token = strtok_n (body, &g_ncDelimiters, &NTX);
-#else
   token = strtok_r (body, DELIMITERS, &ptrptr);
-#endif
   while (token != NULL)
   {
     l = strlen (token);
     if (l >= 1 && l < 50)
     {
-      /* Process "current" token */ 
+#ifdef VERBOSE
+        LOGDEBUG ("Processing body token '%s'", token);
+#endif
+
+      /* Process "current" token */
       if ( !_ds_process_body_token(CTX, token, previous_token, diction)
         && tokenizer == DSZ_CHAIN)
       {
         previous_token = token;
       }
-    } 
-#ifdef NCORE
-    token = strtok_n (NULL, NULL, &NTX);
-#else
+    }
     token = strtok_r (NULL, DELIMITERS, &ptrptr);
-#endif
   }
 
+#ifdef VERBOSE
+  LOGDEBUG("Finished tokenizing (ngram) message");
+#endif
+
   /* Final token reassembly (anything left in the buffer) */
 
   return 0;
 }
 
 int _ds_tokenize_sparse(
-  DSPAM_CTX *CTX, 
-  char *headers, 
-  char *body, 
+  DSPAM_CTX *CTX,
+  char *headers,
+  char *body,
   ds_diction_t diction)
 {
   int i;
   char *token;				/* current token */
   char *previous_tokens[SPARSE_WINDOW_SIZE];	/* sparse chain */
-#ifdef NCORE
-  nc_strtok_t NTX;
-#endif
 
   char *line = NULL;			/* header broken up into lines */
   char *ptrptr;
@@ -320,6 +303,10 @@ int _ds_tokenize_sparse(
   while (node_nt) {
     int multiline;
 
+#ifdef VERBOSE
+    LOGDEBUG("processing line: %s", node_nt->ptr);
+#endif
+
     _ds_sparse_clear(previous_tokens);
 
     line = node_nt->ptr;
@@ -357,7 +344,7 @@ int _ds_tokenize_sparse(
     /* Received headers use a different set of delimiters to preserve things
        like ip addresses */
 
-    token = strtok_r ((multiline) ? line : NULL, DELIMITERS_HEADING, &ptrptr);
+    token = strtok_r ((multiline) ? line : NULL, SPARSE_DELIMITERS_HEADING, &ptrptr);
 
     while (token)
     {
@@ -371,7 +358,7 @@ int _ds_tokenize_sparse(
         _ds_map_header_token (CTX, token, previous_tokens, diction, heading, bitpattern);
       }
 
-      token = strtok_r (NULL, DELIMITERS_HEADING, &ptrptr);
+      token = strtok_r (NULL, SPARSE_DELIMITERS_HEADING, &ptrptr);
     }
 
     for(i=0;i<SPARSE_WINDOW_SIZE;i++) {
@@ -387,34 +374,38 @@ int _ds_tokenize_sparse(
    * Body Tokenization
    */
 
-#ifdef NCORE
-  token = strtok_n (body, &g_ncDelimiters, &NTX);
-#else
-  token = strtok_r (body, DELIMITERS, &ptrptr);
+#ifdef VERBOSE
+  LOGDEBUG("parsing message body");
 #endif
+
+  token = strtok_r (body, SPARSE_DELIMITERS, &ptrptr);
   while (token != NULL)
   {
     l = strlen (token);
     if (l > 0 && l < 50)
     {
-      /* Process "current" token */ 
-      _ds_map_body_token (CTX, token, previous_tokens, diction, bitpattern);
-    } 
-
-#ifdef NCORE
-    token = strtok_n (NULL, NULL, &NTX);
-#else
-    token = strtok_r (NULL, DELIMITERS, &ptrptr);
+#ifdef VERBOSE
+        LOGDEBUG ("Processing body token '%s'", token);
 #endif
+
+      /* Process "current" token */
+      _ds_map_body_token (CTX, token, previous_tokens, diction, bitpattern);
+    }
+    token = strtok_r (NULL, SPARSE_DELIMITERS, &ptrptr);
   }
 
   for(i=0;i<SPARSE_WINDOW_SIZE;i++) {
     _ds_map_body_token(CTX, NULL, previous_tokens, diction, bitpattern);
   }
 
-  _ds_sparse_clear(previous_tokens); 
+  _ds_sparse_clear(previous_tokens);
 
   free(bitpattern);
+
+#ifdef VERBOSE
+  LOGDEBUG("Finished tokenizing (sparse) message");
+#endif
+
   return 0;
 }
 
@@ -473,8 +464,10 @@ _ds_process_header_token (DSPAM_CTX * CTX, char *token,
     char *tweaked_previous;
 
     tweaked_previous = _ds_truncate_token(previous_token);
-    if (tweaked_previous == NULL)
+    if (tweaked_previous == NULL) {
+      free(tweaked_token);
       return EUNKNOWN;
+    }
 
     snprintf (combined_token, sizeof (combined_token),
               "%s*%s+%s", heading, tweaked_previous, tweaked_token);
@@ -507,8 +500,10 @@ _ds_process_body_token (DSPAM_CTX * CTX, char *token,
   if (CTX->tokenizer == DSZ_CHAIN && previous_token != NULL)
   {
     char *tweaked_previous = _ds_truncate_token(previous_token);
-    if (tweaked_previous == NULL)
+    if (tweaked_previous == NULL) {
+      free(tweaked_token);
       return EUNKNOWN;
+    }
 
     snprintf (combined_token, sizeof (combined_token), "%s+%s",
               tweaked_previous, tweaked_token);
@@ -555,7 +550,7 @@ _ds_map_header_token (DSPAM_CTX * CTX, char *token,
   breadth = _ds_pow2(active);
   
   /* Iterate and generate all keys necessary */
-  for (mask=0; mask < breadth; mask++) {
+  for (mask=0; mask < (u_int32_t)breadth; mask++) {
     int terms = 0;
 
     key[0] = 0;
@@ -567,7 +562,7 @@ _ds_map_header_token (DSPAM_CTX * CTX, char *token,
     for(i=0;i<SPARSE_WINDOW_SIZE;i++) {
 
       if (t) {
-        if (keylen < (sizeof(key)-1)) {
+        if ((size_t)keylen < (sizeof(key)-1)) {
           key[keylen] = '+';
           key[++keylen] = 0;
         }
@@ -575,7 +570,7 @@ _ds_map_header_token (DSPAM_CTX * CTX, char *token,
 
       if (bitpattern[(mask*SPARSE_WINDOW_SIZE) + i] == 1) {
         if (previous_tokens[i] == NULL || previous_tokens[i][0] == 0) {
-          if (keylen < (sizeof(key)-1)) {
+          if ((size_t)keylen < (sizeof(key)-1)) {
             key[keylen] = '#';
             key[++keylen] = 0;
           }
@@ -583,14 +578,14 @@ _ds_map_header_token (DSPAM_CTX * CTX, char *token,
         else
         {
           int tl = strlen(previous_tokens[i]);
-          if (keylen + tl < (sizeof(key)-1)) {
+          if ((size_t)(keylen + tl) < (sizeof(key)-1)) {
             strcpy(key+keylen, previous_tokens[i]);
             keylen += tl;
           }
           terms++;
         }
       } else {
-        if (keylen < (sizeof(key)-1)) {
+        if ((size_t)keylen < (sizeof(key)-1)) {
           key[keylen] = '#';
           key[++keylen] = 0;
         }
@@ -655,7 +650,7 @@ _ds_map_body_token (
 
   /* Iterate and generate all keys necessary */
 
-  for(mask=0;mask < breadth;mask++) {
+  for(mask=0;mask < (u_int32_t)breadth;mask++) {
     int terms = 0;
     t = 0;
 
@@ -666,14 +661,14 @@ _ds_map_body_token (
     /* Each Bit */
     for(i=0;i<SPARSE_WINDOW_SIZE;i++) {
       if (t) {
-        if (keylen < (sizeof(key)-1)) {
+        if ((size_t)keylen < (sizeof(key)-1)) {
            key[keylen] = '+';
            key[++keylen] = 0;
         }
       }
       if (bitpattern[(mask*SPARSE_WINDOW_SIZE) + i] == 1) {
         if (previous_tokens[i] == NULL || previous_tokens[i][0] == 0) {
-          if (keylen < (sizeof(key)-1)) {
+          if ((size_t)keylen < (sizeof(key)-1)) {
             key[keylen] = '#';
             key[++keylen] = 0;
           }
@@ -681,14 +676,14 @@ _ds_map_body_token (
         else
         {
           int tl = strlen(previous_tokens[i]);
-          if (keylen + tl < (sizeof(key)-1)) {
+          if ((size_t)(keylen + tl) < (sizeof(key)-1)) {
             strcpy(key+keylen, previous_tokens[i]);
             keylen += tl;
           }
           terms++;
         }
       } else {
-        if (keylen < (sizeof(key)-1)) {
+        if ((size_t)keylen < (sizeof(key)-1)) {
           key[keylen] = '#';
           key[++keylen] = 0;
         }
@@ -721,7 +716,7 @@ _ds_map_body_token (
   return 0;
 }
 
-/* 
+/*
  *  _ds_degenerate_message()
  *
  * DESCRIPTION
@@ -740,15 +735,15 @@ _ds_map_body_token (
 
 int _ds_degenerate_message(DSPAM_CTX *CTX, buffer * header, buffer * body)
 {
-  char *decode, *x, *y;
+  char *decode = NULL;
   struct nt_node *node_nt, *node_header;
   struct nt_c c_nt, c_nt2;
   int i = 0;
   char heading[1024];
 
-  if (! CTX->message) 
+  if (! CTX->message)
   {
-    LOG (LOG_WARNING, "_ds_actualize_message() failed: CTX->message is NULL");
+    LOG (LOG_WARNING, "_ds_degenerate_message() failed: CTX->message is NULL");
     return EUNKNOWN;
   }
 
@@ -770,7 +765,7 @@ int _ds_degenerate_message(DSPAM_CTX *CTX, buffer * header, buffer * body)
 #endif
     }
 
-    else 
+    else
     {
       struct _ds_header_field *current_header;
 
@@ -796,7 +791,7 @@ int _ds_degenerate_message(DSPAM_CTX *CTX, buffer * header, buffer * body)
         /* Accumulate the bodies, skip attachments */
 
         if (
-             (   block->encoding == EN_BASE64 
+             (   block->encoding == EN_BASE64
               || block->encoding == EN_QUOTED_PRINTABLE)
             && ! block->original_signed_body)
         {
@@ -807,126 +802,41 @@ int _ds_degenerate_message(DSPAM_CTX *CTX, buffer * header, buffer * body)
             decode = _ds_decode_block (block);
           }
         }
-  
+
         /* We found a tokenizable body component, add prefilters */
 
         if (decode)
         {
-          char *decode2 = strdup(decode);
-          size_t len = strlen(decode2) + 1;
-  
+          char *decode2 = NULL;
+          char *decode3 = NULL;
+
           /* -- PREFILTERS BEGIN -- */
-  
+
           /* Hexadecimal 8-Bit Encodings */
 
           if (block->encoding == EN_8BIT) {
-            char hex[5] = "0x00";
-            int conv;
-
-            x = strchr(decode2, '%');
-            while(x) {
-              if (isxdigit((unsigned char) x[1]) && 
-                  isxdigit((unsigned char) x[2])) 
-               {
-                hex[2] = x[1];
-                hex[3] = x[2];
-
-                conv = strtol(hex, NULL, 16);
-                if (conv) {
-                  x[0] = conv;
-                  memmove(x+1, x+3, len-((x+3)-decode2));
-                  len -= 2;
-                }
-              }
-              x = strchr(x+1, '%');
-            }
+            decode2 = _ds_decode_hex8bit(decode);
+          } else {
+            decode2 = strdup(decode);
           }
 
           /* HTML-Specific Filters */
-  
-          if (block->media_subtype == MST_HTML) {
-  
-            /* Remove long HTML Comments */
-            x = strstr (decode2, "<!--");
-            while (x != NULL)
-            {
-              y = strstr (x, "-->");
-              if (y)
-              {
-                memmove(x, y + 3, len-((y+3)-decode2)); //strlen (y + 3) + 1);
-                len -= ((y+3) - x);
-                x = strstr (x, "<!--");
-              }
-              else
-              {
-                x = strstr (x + 4, "<!--");
-              }
-            }
 
-            /* Remove short HTML Comments */
-            x = strstr (decode2, "<!");
-            while (x != NULL)
-            {
-              y = strchr (x, '>');
-              if (y != NULL)
-              {
-                memmove(x, y + 1, len-((y+1)-decode2)); //strlen (y + 1) + 1);
-                len -= ((y+1) - x);
-                x = strstr (x, "<!");
-              }
-              else
-              {
-                x = strstr (x + 2, "<!");
-              }
-            }
-
-            /* Remove short HTML tags and those we want to ignore */
-            x = strchr (decode2, '<');
-            while (x != NULL)
-            {
-              int erase = 0;
-              y = strchr (x, '>');
-              if (y != NULL) {
-                if (y - x <= 15
-                      || x[1] == '/'
-                      || !strncasecmp (x + 1, "td ", 3)
-                      || !strncasecmp (x + 1, "table ", 6)
-                      || !strncasecmp (x + 1, "tr ", 3)
-                      || !strncasecmp (x + 1, "div ", 4)
-                      || !strncasecmp (x + 1, "p ", 2)
-                      || !strncasecmp (x + 1, "body ", 5)
-                      || !strncasecmp (x + 1, "!doctype", 8)
-                      || !strncasecmp (x + 1, "blockquote", 10))
-                {
-                  erase = 1;
-                }
-
-                if (!erase) {
-                  char *p = strchr (x, ' ');
-                  if (!p || p > y) 
-                    erase = 1;
-                }
-              }
-              if (erase)
-              {
-                memmove(x, y + 1, len-((y+1)-decode2)); //strlen (y + 1) + 1);
-                len -= ((y+1) - x);
-                x = strstr (x, "<");
-              }
-              else
-              {
-                if (y) 
-                  x = strstr (y + 1, "<");
-                else
-                  x = strstr (x + 1, "<");
-              }
+          if (decode2) {
+            if (block->media_subtype == MST_HTML) {
+              decode3 = _ds_strip_html(decode2);
+            } else {
+              decode3 = strdup(decode2);
             }
+            free(decode2);
           }
 
           /* -- PREFILTERS END -- */
-  
-          buffer_cat (body, decode2);
-          free(decode2);
+
+          if (decode3) {
+            buffer_cat (body, decode3);
+            free(decode3);
+          }
 
           /* If we've decoded the body, save the original copy */
           if (decode != block->body->data)
@@ -938,6 +848,9 @@ int _ds_degenerate_message(DSPAM_CTX *CTX, buffer * header, buffer * body)
         }
       }
     }
+#ifdef VERBOSE
+    LOGDEBUG ("Getting next message component");
+#endif
     node_nt = c_nt_next (CTX->message->components, &c_nt);
     i++;
   } /* while (node_nt != NULL) */
@@ -1047,12 +960,13 @@ void _ds_sparse_clear(char **previous_tokens) {
 
 char *_ds_generate_bitpattern(int breadth) {
   char *bitpattern;
-  unsigned long mask, exp;
+  u_int32_t mask;
+  unsigned long exp;
   int i;
 
   bitpattern = malloc(SPARSE_WINDOW_SIZE * breadth);
 
-  for(mask=0;mask<breadth;mask++) {
+  for(mask=0;mask<(u_int32_t)breadth;mask++) {
       for(i=0;i<SPARSE_WINDOW_SIZE;i++) {
           exp = (i) ? _ds_pow2(i) : 1;
           /* Reverse pos = SPARSE_WINDOW_SIZE - (i+1); */
