@@ -1,22 +1,21 @@
-/* $Id: dspam_pg2int8.c,v 1.11 2006/05/13 01:13:01 jonz Exp $ */
+/* $Id: dspam_pg2int8.c,v 1.15 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -231,6 +230,7 @@ GenSQL (PGconn *dbh,const char *file)
     fprintf(stderr, "Failed to run result: %s\n", PQresultErrorMessage(result));
     if (result) PQclear(result);
     PQfinish(dbh);
+    fclose (out);
     exit(EXIT_FAILURE);
   }
 
@@ -253,6 +253,7 @@ GenSQL (PGconn *dbh,const char *file)
            "I have got no clue of how to deal with this and I am going to sulk now.\n");
         if (result) PQclear(result);
         PQfinish(dbh);
+        fclose (out);
         exit(EXIT_FAILURE);
       }
     }
@@ -302,6 +303,7 @@ GenSQL (PGconn *dbh,const char *file)
                "CREATE INDEX id_token_data_04 ON dspam_token_data(uid);\n"
                "COMMIT;\n"
                "ANALYSE;\n");
+  fclose (out);
 }
 
 /*
