@@ -1,22 +1,21 @@
-/* $Id: dspam_clean.c,v 1.22 2006/05/13 01:13:01 jonz Exp $ */
+/* $Id: dspam_clean.c,v 1.247 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -88,15 +87,22 @@ main (int argc, char *argv[])
   agent_config = read_config(NULL);
   if (!agent_config) {
     LOG(LOG_ERR, ERR_AGENT_READ_CONFIG);
+    fprintf (stderr, ERR_AGENT_READ_CONFIG "\n");
     exit(EXIT_FAILURE);
   }
                                                                                 
   if (!_ds_read_attribute(agent_config, "Home")) {
     LOG(LOG_ERR, ERR_AGENT_DSPAM_HOME);
+    fprintf (stderr, ERR_AGENT_DSPAM_HOME "\n");
     goto bail;
   }
                                                                                 
-  libdspam_init(_ds_read_attribute(agent_config, "StorageDriver"));
+  if (libdspam_init(_ds_read_attribute(agent_config, "StorageDriver")) != 0) {
+    LOG(LOG_ERR, ERR_DRV_INIT);
+    fprintf (stderr, ERR_DRV_INIT "\n");
+    _ds_destroy_config(agent_config);
+    exit(EXIT_FAILURE);
+  }
 
 #ifndef _WIN32
 #ifdef TRUSTED_USER_SECURITY
@@ -113,6 +119,7 @@ main (int argc, char *argv[])
     {
       if (!_ds_match_attribute(agent_config, "Profile", argv[i]+10)) {
         LOG(LOG_ERR, ERR_AGENT_NO_SUCH_PROFILE, argv[i]+10);
+        fprintf (stderr, ERR_AGENT_NO_SUCH_PROFILE "\n", argv[i]+10);
         goto bail;
       } else {
         _ds_overwrite_attribute(agent_config, "DefaultProfile", argv[i]+10);
@@ -122,7 +129,7 @@ main (int argc, char *argv[])
   }
 
 #ifdef DEBUG
-  fprintf (stderr, "dspam_clean starting\n");
+  fprintf (stdout, "dspam_clean starting\n");
 #endif
 
   if (_ds_read_attribute(agent_config, "PurgeSignatures") &&
@@ -223,6 +230,7 @@ main (int argc, char *argv[])
     set_libdspam_attributes(CTX);
     if (dspam_attach(CTX, NULL)) {
       LOG (LOG_WARNING, "unable to attach dspam context");
+      fprintf (stderr, "Unable to attach DSPAM context\n");
       goto bail;
     }
 
@@ -253,6 +261,7 @@ main (int argc, char *argv[])
     set_libdspam_attributes(CTX2);
     if (dspam_attach(CTX2, NULL)) {
       LOG (LOG_WARNING, "unable to attach dspam context");
+      fprintf (stderr, "Unable to attach DSPAM context\n");
       goto bail;
     }
 
@@ -268,11 +277,13 @@ main (int argc, char *argv[])
     if (users->items == 0) {
       user = _ds_get_nextuser (CTX);
     } else {
-      node = node->next;
-      if (node == NULL)
+      if (node == NULL || node->next == NULL) {
+        node = NULL;
         user = NULL;
-      else
+      } else {
+        node = node->next;
         user = node->ptr;
+      }
     }
   }
 
@@ -460,6 +471,7 @@ int process_unused (DSPAM_CTX *CTX, int any, int quota, int nospam, int onehit)
 void
 dieout (int signal)
 {
+  signal = signal; /* Keep compiler happy */
   fprintf (stderr, "terminated.\n");
   if (open_ctx != NULL)
     dspam_destroy (open_ctx);
