@@ -1,22 +1,21 @@
-/* $Id: dspam_dump.c,v 1.13 2006/05/13 01:13:01 jonz Exp $ */
+/* $Id: dspam_dump.c,v 1.21 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -51,6 +50,11 @@
 #include "config_api.h"
 #include "language.h"
 
+#ifdef _WIN32
+/* no trusted users under Windows */
+#undef TRUSTED_USER_SECURITY
+#endif
+
 DSPAM_CTX *open_ctx;
 
 /* headers */
@@ -58,7 +62,7 @@ int dump_database (DSPAM_CTX * CTX, const char *token, int sql);
 void dieout (int signal);
 
 #define TSYNTAX	\
-  "syntax: dspam_dump [--profile=Profile] [-d sqlite_drv] [username [token]]"
+  "syntax: dspam_dump [--profile=Profile] [-d sqlite_drv] username [token]"
 
 #define SQL_SQLITE_DRV	1
 
@@ -68,10 +72,8 @@ main (int argc, char **argv)
   DSPAM_CTX *CTX;
   int r, sql = 0, i;
   char *username = NULL, *token = NULL;
-#ifndef _WIN32
 #ifdef TRUSTED_USER_SECURITY
   struct passwd *p = getpwuid (getuid ());
-#endif
 #endif
 
   /* Read dspam.conf */
@@ -79,24 +81,22 @@ main (int argc, char **argv)
   agent_config = read_config(NULL);
   if (!agent_config) {
     LOG(LOG_ERR, ERR_AGENT_READ_CONFIG);
+    fprintf (stderr, ERR_AGENT_READ_CONFIG "\n");
     exit(EXIT_FAILURE);
   }
                                                                                 
   if (!_ds_read_attribute(agent_config, "Home")) {
     LOG(LOG_ERR, ERR_AGENT_DSPAM_HOME);
+    fprintf (stderr, ERR_AGENT_DSPAM_HOME "\n");
     exit(EXIT_FAILURE);
   }
 
-  libdspam_init(_ds_read_attribute(agent_config, "StorageDriver"));
-
-#ifndef WIN32
-#ifdef TRUSTED_USER_SECURITY
-  if (!_ds_match_attribute(agent_config, "Trust", p->pw_name) && p->pw_uid) {
-    fprintf(stderr, ERR_TRUSTED_MODE "\n");
-    goto BAIL;
+  if (libdspam_init(_ds_read_attribute(agent_config, "StorageDriver")) != 0) {
+    LOG(LOG_ERR, ERR_DRV_INIT);
+    fprintf (stderr, ERR_DRV_INIT "\n");
+    _ds_destroy_config(agent_config);
+    exit(EXIT_FAILURE);
   }
-#endif
-#endif
 
   open_ctx = NULL;
   signal (SIGINT, dieout);
@@ -115,6 +115,7 @@ main (int argc, char **argv)
     {
       if (!_ds_match_attribute(agent_config, "Profile", argv[i]+10)) {
         LOG(LOG_ERR, ERR_AGENT_NO_SUCH_PROFILE, argv[i]+10);
+        fprintf (stderr, ERR_AGENT_NO_SUCH_PROFILE "\n", argv[i]+10);
         goto BAIL;
       } else {
         _ds_overwrite_attribute(agent_config, "DefaultProfile", argv[i]+10);
@@ -131,8 +132,17 @@ main (int argc, char **argv)
         goto BAIL;
       }
     } else {
-      if (username == NULL)
+      if (username == NULL) {
         username = argv[i];
+#ifdef TRUSTED_USER_SECURITY
+        if ( strcmp(username, p->pw_name) &&
+              !_ds_match_attribute(agent_config, "Trust", p->pw_name) &&
+                p->pw_uid) {
+          fprintf(stderr, ERR_TRUSTED_MODE "\n");
+          goto BAIL;
+        }
+#endif
+      }
       else
         token = argv[i];
     }
@@ -151,6 +161,7 @@ main (int argc, char **argv)
   set_libdspam_attributes(CTX);
   if (dspam_attach(CTX, NULL)) {
     LOG (LOG_WARNING, "unable to attach dspam context");
+    fprintf (stderr, "Unable to attach DSPAM context\n");
     goto BAIL;
   }
 
@@ -227,6 +238,7 @@ dump_database (DSPAM_CTX * CTX, const char *token, int sql)
 void
 dieout (int signal)
 {
+  signal = signal; /* Keep compiler happy */
   fprintf (stderr, "terminated.\n");
   if (open_ctx != NULL)
     dspam_destroy (open_ctx);
