@@ -1,22 +1,21 @@
-/* $Id: dspam_stats.c,v 1.25 2006/05/13 01:13:01 jonz Exp $ */
+/* $Id: dspam_stats.c,v 1.36 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -45,7 +44,12 @@
 #include "language.h"
 #include "util.h"
 
-#define TSYNTAX	"syntax: dspam_stats [-hHrsSt] [username]"
+#define TSYNTAX	"syntax: dspam_stats [-h]\|[--profile=PROFILE] [-HrsSt] [user [user...]]"
+
+#ifdef _WIN32
+/* no trusted users under Windows */
+#undef TRUSTED_USER_SECURITY
+#endif
 
 DSPAM_CTX *open_ctx, *open_mtx;
 int opt_humanfriendly;
@@ -68,11 +72,9 @@ main (int argc, char **argv)
 #endif
   struct _ds_spam_totals totals;
 
-#ifndef _WIN32
 #ifdef TRUSTED_USER_SECURITY
   struct passwd *p = getpwuid (getuid ());
-                                                                                
-#endif
+  int trusted = 0;                                                                                
 #endif
 
 
@@ -83,32 +85,44 @@ main (int argc, char **argv)
   agent_config = read_config(NULL);
   if (!agent_config) {
     LOG(LOG_ERR, ERR_AGENT_READ_CONFIG);
+    fprintf (stderr, ERR_AGENT_READ_CONFIG "\n");
     exit(EXIT_FAILURE);
   }
                                                                                 
   if (!_ds_read_attribute(agent_config, "Home")) {
     LOG(LOG_ERR, ERR_AGENT_DSPAM_HOME);
+    fprintf (stderr, ERR_AGENT_DSPAM_HOME "\n");
     _ds_destroy_config(agent_config);
     exit(EXIT_FAILURE);
   }
                                                                                 
-  libdspam_init(_ds_read_attribute(agent_config, "StorageDriver"));
-
-#ifndef _WIN32
-#ifdef TRUSTED_USER_SECURITY
-  if (!_ds_match_attribute(agent_config, "Trust", p->pw_name) && p->pw_uid) {
-    fprintf(stderr, ERR_TRUSTED_MODE "\n");
+  if (libdspam_init(_ds_read_attribute(agent_config, "StorageDriver")) != 0) {
+    LOG(LOG_ERR, ERR_DRV_INIT);
+    fprintf (stderr, ERR_DRV_INIT "\n");
     _ds_destroy_config(agent_config);
-    goto BAIL;
+    exit(EXIT_FAILURE);
+  }
+
+#ifdef TRUSTED_USER_SECURITY
+  if (_ds_match_attribute(agent_config, "Trust", p->pw_name) || !p->pw_uid) {
+    trusted = 1;
   }
-#endif
 #endif
 
   for(i=0;i<argc;i++) {
     if (!strncmp (argv[i], "--profile=", 10))
     {
+#ifdef TRUSTED_USER_SECURITY
+      if (!trusted) {
+        LOG(LOG_ERR, ERR_TRUSTED_PRIV, "--profile", p->pw_uid, p->pw_name);
+        fprintf (stderr, ERR_TRUSTED_PRIV "\n", "--profile", p->pw_uid, p->pw_name);
+        _ds_destroy_config(agent_config);
+        goto BAIL;
+      }
+#endif
       if (!_ds_match_attribute(agent_config, "Profile", argv[i]+10)) {
         LOG(LOG_ERR, ERR_AGENT_NO_SUCH_PROFILE, argv[i]+10);
+        fprintf (stderr, ERR_AGENT_NO_SUCH_PROFILE "\n", argv[i]+10);
         _ds_destroy_config(agent_config);
         goto BAIL;
       } else {
@@ -132,13 +146,13 @@ main (int argc, char **argv)
   ch = opt_humanfriendly = 0;
   opt_reset = opt_snapshot = opt_stats = opt_total =  0;
 
-#ifdef HAVE_GETOPT
-  while((ch = getopt(argc, argv, "hHrsS")) != -1)
-#else
+#ifndef HAVE_GETOPT
   while ( argv[optind] &&
             argv[optind][0] == '-' &&
               (ch = argv[optind][1]) &&
                 argv[optind][2] == '\0' )
+#else
+  while((ch = getopt(argc, argv, "hHrsS")) != -1)
 #endif
   {
     switch(ch) {
@@ -173,21 +187,42 @@ main (int argc, char **argv)
     optind++;
 #endif
   }
+#ifndef HAVE_GETOPT
   /* reset our option array and index to where we are after getopt */
   argv += optind;
   argc -= optind;
+#endif
 
   /* process arguments */
   for (i=0; i < argc; i++)
   {
-      if (strncmp(argv[i], "--", 2)) {
+      if (argv[i] && strncmp(argv[i], "--", 2)) {
+#ifdef TRUSTED_USER_SECURITY
+        if ( !trusted && strcmp(argv[i], p->pw_name) )
+        {
+          fprintf(stderr, ERR_TRUSTED_MODE "\n");
+          _ds_destroy_config(agent_config);
+          goto BAIL;
+        }
+#endif
         stat_user(argv[i], &totals);
         users++;
       }
   }
 
   if (!users)
+  {
+#ifdef TRUSTED_USER_SECURITY
+    if ( !trusted )
+    {
+      fprintf(stderr, ERR_TRUSTED_MODE "\n");
+      _ds_destroy_config(agent_config);
+      goto BAIL;
+    }
+#endif
+
     process_all_users (&totals);
+  }
 
   if (opt_total)
     stat_user(NULL, &totals);
@@ -218,6 +253,7 @@ process_all_users (struct _ds_spam_totals *totals)
   set_libdspam_attributes(CTX);
   if (dspam_attach(CTX, NULL)) {
     LOG (LOG_WARNING, "unable to attach dspam context");
+    fprintf (stderr, "Unable to attach DSPAM context\n");
     dspam_destroy(CTX);
     return EFAILURE;
   }
@@ -254,6 +290,7 @@ stat_user (const char *username, struct _ds_spam_totals *totals)
     set_libdspam_attributes(MTX);
     if (dspam_attach(MTX, NULL)) {
       LOG (LOG_WARNING, "unable to attach dspam context");
+      fprintf (stderr, "Unable to attach DSPAM context\n");
       return EUNKNOWN;
     }
     tptr = &MTX->totals;
@@ -320,16 +357,17 @@ stat_user (const char *username, struct _ds_spam_totals *totals)
   if (opt_humanfriendly)
   {
     printf("%s:\n\
-        \tTP True Positives:         %6ld\n\
-        \tTN True Negatives:         %6ld\n\
-        \tFP False Positives:        %6ld\n\
-        \tFN False Negatives:        %6ld\n\
-        \tSC Spam Corpusfed:         %6ld\n\
-        \tNC Nonspam Corpusfed:      %6ld\n\
-        \tTL Training Left:          %6ld\n\
-        \tSHR Spam Hit Rate        % 7.2f%%\n\
-        \tHSR Ham Strike Rate:     % 7.2f%%\n\
-        \tOCA Overall Accuracy:    % 7.2f%%\n\
+        \tTP True Positives:                %6ld\n\
+        \tTN True Negatives:                %6ld\n\
+        \tFP False Positives:               %6ld\n\
+        \tFN False Negatives:               %6ld\n\
+        \tSC Spam Corpusfed:                %6ld\n\
+        \tNC Nonspam Corpusfed:             %6ld\n\
+        \tTL Training Left:                 %6ld\n\
+        \tSHR Spam Hit Rate               % 7.2f%%\n\
+        \tHSR Ham Strike Rate:            % 7.2f%%\n\
+        \tPPV Positive predictive value:  % 7.2f%%\n\
+        \tOCA Overall Accuracy:           % 7.2f%%\n\
         \n",
             (username) ? username : "TOTAL",
             total_spam, total_innocent,
@@ -343,6 +381,10 @@ stat_user (const char *username, struct _ds_spam_totals *totals)
         (all_innocent) ?
           100-(100.0-((float)innocent_misclassified / (float)all_innocent )*100.0)
           : 100.0,
+        (total_spam + innocent_misclassified) ?
+          100-(100.0-((float)total_spam /
+                  (float)(total_spam + innocent_misclassified))*100)
+          : 100.0,
         (all_spam + all_innocent) ?
           (100.0-(((float)spam_misclassified +(float)innocent_misclassified) /
                    (float)(all_spam + all_innocent))*100.0)
@@ -416,6 +458,7 @@ stat_user (const char *username, struct _ds_spam_totals *totals)
 void
 dieout (int signal)
 {
+  signal = signal; /* Keep compile happy */
   fprintf (stderr, "terminated.\n");
   if (open_ctx != NULL)
     dspam_destroy (open_ctx);
@@ -429,11 +472,15 @@ void
 usage (void)
 {
   (void)fprintf (stderr,
-    "usage: dspam_stats [-hH] [user [user...]]\n\
+    "usage: dspam_stats [-h]|[--profile=PROFILE] [-HrsSt] [user [user...]]\n\
       \tPrint dspam statistics for users.\n\
       \tIf no users are specified, stats for all users are printed.\n\
       \t-h: print this message\n\
-      \t-H: print stats in \"human friendly\" format\n");
+      \t-H: print stats in \"human friendly\" format\n\
+      \t-r: Resets the current snapshot\n\
+      \t-s: Displays stats since last snapshot (instead of since epoch)\n\
+      \t-S: Displays accuracy percentages in addition to stats\n\
+      \t-t: Displays a total of all statistics displayed\n");
   _ds_destroy_config(agent_config);
   exit(EXIT_FAILURE);
 }
