@@ -1,22 +1,21 @@
-/* $Id: util.c,v 1.20 2006/05/13 01:12:59 jonz Exp $ */
+/* $Id: util.c,v 1.269 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -40,6 +39,7 @@
 #include <stdio.h>
 #include <math.h>
 #include <fcntl.h>
+#include <signal.h>
 
 #ifdef TIME_WITH_SYS_TIME
 #   include <sys/time.h>
@@ -428,8 +428,13 @@ _ds_prepare_path_for (const char *filename)
     /* don't try to create root directory of a drive */
     if ( path[2] != '\0' || path[1] != ':' )
 #endif
-    {
-      if (dir != NULL && stat (path, &s) && path[0] != 0)
+   {
+#ifdef EXT_LOOKUP
+	  /* don't create users data dir if user verification is required */
+      if (dir != NULL && stat (path, &s) && path[0] != 0 && verified_user == 1)
+#else
+	  if (dir != NULL && stat (path, &s) && path[0] != 0)
+#endif
       {
         int x;
         LOGDEBUG ("creating directory '%s'", path);
@@ -525,53 +530,136 @@ int _ds_compute_weight(const char *token) {
   int complexity = _ds_compute_complexity(token);
   int sparse = _ds_compute_sparse(token);
 
-  if (complexity == 1 && sparse == 0) /* the */
-    return 1;
-  if (complexity == 2 && sparse == 0) /* the quick */
-    return 4;
+/*
+ * Mathematically correct algorithm (but slower):
+ *
+ * int weight = 0;
+ *
+ * if (complexity >= 1 && complexity <= SPARSE_WINDOW_SIZE) {
+ *   weight = (int)pow(2.0,(2*(complexity-sparse-1)));
+ *   if (weight < 1)
+ *     return 1;
+ *   else
+ *     return weight;
+ * }
+ */
 
-  if (complexity == 3) { 
-    if (sparse == 1) /*		the * brown			*/
-      return 4;
 
-    if (sparse == 0) /*		the quick brown			*/
-      return 16;
-  }
-
-  if (complexity == 4) {
-    if (sparse == 2) /* 	the * * fox			*/
-      return 4;
-    if (sparse == 1) /* 	the quick * fox			*/
-      return 16;
-    if (sparse == 0) /*		the quick brown fox		*/
-      return 64;
-  }
+/*
+ * The same (+/-) as above but without using an algorithm (and
+ * therefore faster then calling each time the pow() function).
+ * Using reverse order of complexity to speed up processing.
+ *
+ */
 
   if (complexity == 5) {
-    if (sparse == 3) /*		the * * * jumped		*/
-      return 4;
-    if (sparse == 2) /*		the quick * * jumped		*/
-      return 16;
-    if (sparse == 1) /*		the quick brown * jumped	*/
+    if (sparse == 1)  /*  the * brown fox jumped | the quick * fox jumped | the quick brown * jumped  */
       return 64;
-    if (sparse == 0) /*		the quick brown fox jumped	*/
+    if (sparse == 2)  /*  the * * fox jumped | the * brown * jumped | the quick * * jumped  */
+      return 16;
+    if (sparse == 3)  /*  the * * * jumped  */
+      return 4;
+    if (sparse == 0)  /*  the quick brown fox jumped  */
       return 256;
   }
 
-  LOG(LOG_WARNING, "no rule to compute markovian weight for '%s'; complexity: %d; sparse: %d", token, complexity, sparse);
+  if (complexity == 4) {
+    if (sparse == 1)  /*  quick * fox jumped | quick brown * jumped  */
+      return 16;
+    if (sparse == 2)  /*  quick * * jumped  */
+      return 4;
+    if (sparse == 0)  /*  quick brown fox jumped  */
+      return 64;
+  }
+
+  if (complexity == 3) {
+    if (sparse == 1)  /*  brown * jumped  */
+      return 4;
+    if (sparse == 0)  /*  brown fox jumped  */
+      return 16;
+  }
+
+  if (complexity == 2) {
+    if (sparse == 0)  /*  fox jumped  */
+      return 4;
+  }
+
+  if (complexity == 1) {
+    if (sparse == 0)  /*  jumped  */
+      return 1;
+  }
+
+  LOG(LOG_WARNING, "_ds_compute_weight: no rule to compute markovian weight for '%s'; complexity: %d; sparse: %d", token, complexity, sparse);
+  return 1;
+}
+int _ds_compute_weight_osb(const char *token) {
+/* We have two possibilities here to compute the weight.
+ *
+ * One would be to use the original code found in older
+ * CRM114 and compute (have larger weights for 'shorter'
+ * (narrower) matches):
+ *   (SPARSE_WINDOW_SIZE-sparse)**(SPARSE_WINDOW_SIZE-sparse)
+ *
+ * Or use newer algorithm found in CRM114 (have larger
+ * weights for 'longer' (wider) matches):
+ *   complexity**complexity
+ *
+ * We are going to use here the later one.
+ */
+
+  int complexity = _ds_compute_complexity(token);
+
+/*
+ * Mathematically correct algorithm (but slower):
+ *
+ * int weight = 1;
+ *
+ * if (complexity >= 1 && complexity <= SPARSE_WINDOW_SIZE) {
+ *   weight = (int)pow(complexity,complexity);
+ *   if (weight < 1)
+ *     weight = 1;
+ * }
+ * return weight;
+ */
+
+/*
+ * The same (+/-) as above but without using an algorithm (and
+ * therefore faster then calling each time the pow() function).
+ *
+ */
+
+  if (complexity == 5) {  /*  the * * * jumped  */
+    return 3125;
+  }
+
+  if (complexity == 4) {  /*  quick * * jumped  */
+    return 256;
+  }
+
+  if (complexity == 3) {  /*  brown * jumped  */
+    return 27;
+  }
+
+  if (complexity == 2) {  /*  fox jumped  */
+    return 4;
+  }
+
+  LOG(LOG_WARNING, "_ds_compute_weight_osb: no rule to compute OSB/OSBF/WINNOW weight for '%s'; complexity: %d", token, complexity);
   return 1;
 }
 
 int _ds_compute_sparse(const char *token) {
   int sparse = 0, i;
 
- if (!strncmp(token, "#+", 2))
+  if (!strncmp(token, "#+", 2))
     sparse++;
   if (strlen(token)>=2 && !strncmp((token+strlen(token))-2, "+#", 2))
     sparse++;
   for(i=0;token[i];i++) {
-    if (!strncmp(token+i, "+#+", 3))
+    if (!strncmp(token+i, "+#+", 3)) {
       sparse++;
+      i++;
+    }
   }
 
   return sparse;
@@ -579,13 +667,15 @@ int _ds_compute_sparse(const char *token) {
 
 int _ds_compute_complexity(const char *token) {
   int i, complexity = 1;
-                                                                                
+
   if (token == NULL)
     return 1;
-                                                                                
+
   for(i=0;token[i];i++) {
-    if (token[i] == '+')
+    if (token[i] == '+') {
       complexity++;
+      i++;
+    }
   }
 
   return complexity;
@@ -629,18 +719,33 @@ double chi2Q (double x, int v)
   return MIN(s, 1.0);
 }
 
+void timeout(void) {}
+
 int _ds_get_fcntl_lock(int fd) {
 #ifdef _WIN32
   return 0;
 #else
   struct flock f;
+  int r;
 
   f.l_type = F_WRLCK;
   f.l_whence = SEEK_SET;
   f.l_start = 0;
   f.l_len = 0;
 
-  return fcntl(fd, F_SETLKW, &f);
+#if defined __GLIBC__ && __GLIBC__ >= 2
+  signal(SIGALRM, (sighandler_t)timeout);
+#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
+  signal(SIGALRM, (sig_t)timeout);
+#else
+  signal(SIGALRM, (void *)timeout);
+#endif
+  alarm(300);
+  r=fcntl(fd, F_SETLKW, &f);
+  alarm(0);
+  signal(SIGALRM,SIG_DFL);
+
+  return r;
 #endif
 }
 
@@ -659,13 +764,19 @@ int _ds_free_fcntl_lock(int fd) {
 #endif
 } 
 
+int _ds_pow(int base, unsigned int exp) {
+  int result = 1;
+  while (exp > 0) {
+    if (exp & 1)
+      result *= base;
+    base *= base;
+    exp /= 2;
+  }
+  return result;
+}
+
 int _ds_pow2(int exp) {
-  int j = 1, i;
-  if (!exp)
-    return j;
-  for(i=0;i<exp;i++)
-    j *= 2;
-  return j;
+  return _ds_pow(2, exp);
 }
 
 float _ds_round(float n) {
@@ -680,13 +791,14 @@ char *
 strcasestr(s, find)
 	const char *s, *find;
 {
-	char c, sc;
-	size_t len;
+	char c;
 
 	if ((c = *find++) != 0) {
+		size_t len;
 		c = tolower((unsigned char)c);
 		len = strlen(find);
 		do {
+			char sc;
 			do {
 				if ((sc = *s++) == 0)
 					return (NULL);
@@ -743,7 +855,7 @@ inet_ntoa_r(struct in_addr in, char *buf, int len)
   snprintf(b, sizeof(b),
   "%d.%d.%d.%d", UC(p[0]), UC(p[1]), UC(p[2]), UC(p[3]));
 
-  if (len <= strlen(b)) {
+  if ((size_t)len <= strlen(b)) {
     errno = ERANGE;
     return(NULL);
   }
@@ -751,3 +863,72 @@ inet_ntoa_r(struct in_addr in, char *buf, int len)
   return strcpy(buf, b);
 }
 #endif
+
+/*
+ * _ds_validate_address() - Validate a email address
+ *
+ * DESCRIPTION
+ *   validate the supplied email address
+ *
+ * INPUT ARGUMENTS
+ *   const char *address      pointer to email address
+ *
+ * NOTES
+ *   This function is NOT RFC 821/822 complete. However... most invalid
+ *   RFC 821/822 email addresses should be caputred by this function.
+ *   Extend this function if you need to capture more invalid email addresses.
+ *
+ * RETURN VALUES
+ *   returns 1 if email address is valid
+ *   returns 0 if email address is not valid
+ *
+ */
+int _ds_validate_address(const char *address) {
+  int count = 0;
+  const char *p;
+  char *email;
+  const char *domain;
+  static char *rfc822_specials = "()<>@,;:\\\"[]";
+
+  /* remove < at the beginning and > at the end of email address */
+  email = ALLTRIM(strdup(address));
+  if (*email == '<' && *(email + strlen(email) - 1) == '>') {
+    *(email + strlen(email) - 1) = 0;
+    email++;
+  }
+
+  /* First validate the local part (local_part@domain_part.tld) */
+  if (*email == '.') return 0;
+  for (p = email;  *p;  p++) {
+    if (*p == '\"' && (p == email || *(p - 1) == '.' || *(p - 1) == '\"')) {
+      while (*++p) {
+        if (*p == '\"') break;
+        if (*p == '\\' && (*++p == ' ')) continue;
+        if (*p < ' ' || *p >= 127) return 0;
+      }
+      if (!*p++) return 0;
+      if (*p == '@') break;
+      if (*p != '.') return 0;
+      continue;
+    }
+    if (*p == '@') break;
+    if (*p == '.' && (*++p == '.')) return 0;
+    if (*p <= ' ' || *p >= 127) return 0;
+    if (strchr(rfc822_specials, *p)) return 0;
+  }
+  if (p == email || *(p - 1) == '.') return 0;
+
+  /* Next validate the domain part (local_part@domain_part.tld) */
+  if (!*(domain = ++p)) return 0;
+  do {
+    if (*p == '.') {
+      if (p == domain || *(p - 1) == '.') return 0;
+      count++;
+    }
+    if (*p <= ' ' || *p >= 127) return 0;
+    if (*p == '.' && (*++p == '.')) return 0;
+    if (strchr(rfc822_specials, *p)) return 0;
+  } while (*++p);
+
+  return (count >= 1);
+}
