@@ -1,22 +1,21 @@
-/* $Id: util.h,v 1.13 2006/05/13 01:12:59 jonz Exp $ */
+/* $Id: util.h,v 1.20 2011/06/28 00:13:48 sbajic Exp $ */
 
 /*
  DSPAM
- COPYRIGHT (C) 2002-2006 JONATHAN A. ZDZIARSKI
+ COPYRIGHT (C) 2002-2011 DSPAM PROJECT
 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; version 2
- of the License.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as
+ published by the Free Software Foundation, either version 3 of the
+ License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ GNU Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
@@ -77,6 +76,9 @@ size_t strlcat (char *, const char *, size_t);
  *
  *  _ds_compute_weight()
  *    Calculates the markovian weight of a token
+ *
+ *  _ds_compute_weight_osb()
+ *    Calculates the OSB/OSBF/WINNOW weight of a token
  */
 
 #ifndef HAVE_STRTOK_R
@@ -90,6 +92,10 @@ char *inet_ntoa_r
   (struct in_addr in, char *buf, int len);
 #endif
 
+#ifdef EXT_LOOKUP
+  int verified_user;
+#endif
+
 const char *	_ds_userdir_path (
   char *buff,
   const char *home,
@@ -100,6 +106,7 @@ int _ds_prepare_path_for   (const char *filename);
 int _ds_compute_complexity (const char *token);
 int _ds_compute_sparse     (const char *token);
 int _ds_compute_weight     (const char *token);
+int _ds_compute_weight_osb (const char *token);
 char *_ds_truncate_token   (const char *token);
 
 int	_ds_extract_address(
@@ -109,11 +116,15 @@ int	_ds_extract_address(
 
 double	_ds_gettime(void);
 
+void timeout(void);
+
 int _ds_get_fcntl_lock  (int fd);
 int _ds_free_fcntl_lock (int fd);
 
 unsigned long long _ds_getcrc64
   (const char *);
+int _ds_pow
+  (int base, unsigned int exp);
 int _ds_pow2
   (int exp);
 double chi2Q
@@ -121,4 +132,6 @@ double chi2Q
 float _ds_round
   (float n);
 
+int _ds_validate_address
+  (const char *address);
 #endif /* _UTIL_H */
